#!/usr/bin/env python
# -*- coding: utf-8 -*-
import argparse,copy,datetime,dendropy,logging,os,re,sys, multiprocessing,msatools,subprocess,platform,stat
import numpy as np
import random as rnd
import settings as sp
from msatools import *
from select import select

try:
	from collections import Counter
except ImportError:
	from counter import Counter

class SequenceGenerator:
	"""
	Class for the generation of genome sequences from gene trees
	----------------------------------------------------------------------------
	Attributes:
	- appLogger: logger to store status of the process flow
	- settings: Settings object withh all the program parameters
	- newIndelibleAncestralSequence: path of the reference.txt file that will be
	 used to call the sequence simulator INDELible.
	- newIndelibleControlFilePath: path of the control file that will be used to call
	the sequence simulator INDELible
	- output: path where the output of the INDELible execution will be stored
	- evolve: default parameters of the [EVOLVE] section of the INDELible control
	file
	- partition: parameters of the [PARTITION] section of the INDELible control
	file
	"""
	appLoger=None
	settings=None

	ancestralFreq=dict({"A":0.25, "C":0.25, "G":0.25,"T":0.25})
	newIndelibleAncestralSequence=""
	newIndelibleControlFilePath=""
	geneTreeFile=""

	evolve=[1,"ngsphydata_1"]
	partition=[]

	def __init__(self,settings):
		self.appLogger=logging.getLogger("ngsphy")
		self.appLogger.debug('INDELible run')
		self.settings=settings
		self.settings.alignmentsFolderPath=os.path.join(self.settings.alignmentsFolderPath,"1")
		self.newIndelibleControlFilePath=os.path.join(\
			self.settings.alignmentsFolderPath,"control.txt")
		self.newIndelibleAncestralSequence=os.path.join(\
			self.settings.alignmentsFolderPath,"ancestral.fasta")
		if self.settings.inputmode==3: # INDELible + reference
			self.geneTreeFile=os.path.join(\
				self.settings.alignmentsFolderPath,\
				"ngsphy.tree"\
			)
		self.generateFolderStructure()

	def run(self):
		"""
		Process flow for the generation of genome sequences from a gene tree
		and an evolutionary model
		"""
		self.writeIndelibleControlFile()

		runStatus,runMessage=self.runIndelible()
		if not runStatus: return False,runMessage
		return True, "Run finished"

	def generateFolderStructure(self):
		"""
		Generation of a folder structure for this process.
		"""
		self.appLogger.info("Creating folder structure for INDELible run")
		try:
			os.makedirs(os.path.join(self.settings.alignmentsFolderPath))
			self.appLogger.info("Generating data folder ({0})".format(\
				os.path.join(self.settings.alignmentsFolderPath)))
		except:
			self.appLogger.debug("Data folder exists ({0})".format(\
				os.path.join(self.settings.alignmentsFolderPath)))


	def copyAncestralSequenceToOutputFolder(self):
		"""
		In order to generate genome sequences all the required files must be
		in the same folder where INDELible is going to be ran. Hence, the need
		of copying the given reference file to the directory where data will
		be stored.
		-----------------------------------------------------------------------
		Returns:
		- boolean. Indicates the status of the process.
		"""
		# making sure there's only one sequence, and only one sequence will be written to the
		# reference.fasta file
		# that sequence will be the first from the file if there are more than 1 sequence
		status=True; message=""
		self.appLogger.debug("Copying reference sequence file ")
		self.appLogger.info("Copying reference sequence file to: {}".format(\
			self.newIndelibleAncestralSequence))
		description=""
		try:
			with open(self.settings.ancestralSequenceFilePath, "r") as f:
				description=f.readline().strip()
		except Exception as ex:
			message="\n\t{0}\n\t{1}\n\t{2}\n\t{3}\n".format(\
				"I/O problem.",\
				ex,
				"Stopped while reading the ancestral sequence file.",\
				"Please verify and rerun. Exiting."
			)
			status=False
			return status, message
		description=description[1:len(description)]
		referenceDict=msatools.parseMSAFileWithDescriptions(self.settings.ancestralSequenceFilePath)
		reference=referenceDict[description]
		self.getAncestralSequenceBaseFrequencies(reference)
		try:
			fout=open(self.newIndelibleAncestralSequence,"w")
			fout.write(">ngsphypartition\n{}\n".format(reference))
			fout.close()
		except Exception as ex:
			message="\n\t{0}\n\t{1}\n\t{2}\n\t{3}\n".format(\
				"I/O problem.",\
				ex,\
				"Stopped while copying the ancestral sequence file.",\
				"Please verify and rerun. Exiting."
			)
			status=False
			return status, message
		return status, message

	def getAncestralSequenceBaseFrequencies(self,reference):
		counter=Counter(reference)
		totalBases=len(reference)
		self.ancestralFreq["A"]=float(int(counter["A"])/float(totalBases))
		self.ancestralFreq["C"]=float(int(counter["C"])/float(totalBases))
		self.ancestralFreq["G"]=float(int(counter["G"])/float(totalBases))
		self.ancestralFreq["T"]=float(int(counter["T"])/float(totalBases))
		self.appLogger.info("Extracting base frequencies: {}".format(self.ancestralFreq))

	def writeIndelibleControlFile(self):
		"""
		Writes the modified INDELible control file into the appropriate
		directory to be able, afterwards, to run INDELible properly.
		"""
		self.appLogger.debug("Writing new control file")
		self.appLogger.debug("Given INDELible control file: ".format(\
			self.settings.indelibleControlFile))
		lines=None
		with open(self.settings.indelibleControlFile,"r") as f:
			lines=[line.strip() for line in f ]
		newlines=copy.copy(lines)
		newlines.reverse()
		controllines=[]
		modelname=""
		while len(newlines)>0:
			line=newlines.pop()
			if "[NGSPHYPARTITION]" in line:
				self.partition=line.split() # ill get 3 elems + label
			if "[MODEL]" in line:
				modelname=line.split()[1]
		del newlines
		newlines=copy.copy(lines)
		for item in newlines:
			if item.strip().startswith("[NGSPHY"):
				break
			controllines+=[item.strip("\n")]

		statesFreqPresent=-1
		# BASE FREQUENCIES - [statefreq] T C A G
		stateFreqEntry=["[statefreq]\t{0} {1} {2} {3}".format(\
			self.ancestralFreq["T"],\
			self.ancestralFreq["C"],\
			self.ancestralFreq["A"],\
			self.ancestralFreq["G"]\
			)]
		for index in range(0,len(controllines)):
			if controllines[index].strip().startswith("[statefreq]"):
				statesFreqPresent=index
				break
		if (statesFreqPresent < 0):
			controllines+=stateFreqEntry

		with open(self.settings.geneTreeFile) as f:
			geneTreeFile=[line.strip() for line in f if not line.strip() == ""]
		geneTreeFile="".join(geneTreeFile)
		geneTreeFile=geneTreeFile.replace("'","")
		# print(geneTreeFile)
		if geneTreeFile[-1]!=";":
			geneTreeFile+=";"
		controllines+=["{0} {1} {2}".format(\
			"[TREE]",\
			"ngsphytree",\
			geneTreeFile
		)]
		controllines+=["{0} {1} [{2} {3} {4}]".format(\
			"[PARTITIONS]",\
			"ngsphypartition",\
			"ngsphytree",\
			modelname,\
			self.partition[3]
		)]

		controllines+=["[EVOLVE]"]
		controllines+=[" {0} {1} {2}".format(\
			"ngsphypartition",\
			self.evolve[0],\
			self.evolve[1]\
		)]

		# full control file, missing checking settings of output and fastaextension
		fastaoutput="[output] FASTA"
		fastaoutputext="[fastaextension] fasta"
		output=[]; outputext=[]
		settings=False
		for indexControl in range(0, len(controllines)):
			data=controllines[indexControl].strip()
			if data=="[SETTINGS]":
				settings=True
			if data.startswith("[output]"):
				ss=data.split()[1]
				output+=[ss.upper()]
			if data.startswith("[fastaextension]"):
				outputext+=[indexControl]
		if not settings:
			controllines.insert(1,"[SETTINGS]")
		if (not "FASTA" in output) and (len(outputext) ==0):
			controllines.insert(2,"[output] FASTA")
			controllines.insert(3,"[fastaextension] fasta")
		elif (not "FASTA" in output):
			controllines.insert(2,"[output] FASTA")
		elif (len(outputext) ==0):
			controllines.insert(2,"[fastaextension] fasta")
		# check whether there is thr [randomseed] options
		# if exists keep, else use the program one
		randomseed=False
		for indexControl in range(0,len(controllines)):
			data=controllines[indexControl].strip()
			if data.startswith("[randomseed]"): randomseed=True
		if not randomseed:
			controllines.insert(2,"[randomseed] {0}".format(self.settings.seed))
		# write controllines to file
		f=open(self.newIndelibleControlFilePath,"w")
		for item in controllines:
			f.write("{}\n".format(item))
		f.close
		st = os.stat(self.newIndelibleControlFilePath)
		os.chmod(self.newIndelibleControlFilePath, st.st_mode | stat.S_IEXEC | stat.S_IXGRP | stat.S_IXOTH)

	def runIndelible(self):
		"""
		Launches a thread with the INDELible command.
		------------------------------------------------------------------------
		Returns:
		- boolean. Indicates status of the process
		"""
		self.appLogger.debug("Running...")
		self.settings.parser.set("general","numLociPerReplicate",str(1))
		self.settings.parser.set("general", "filtered_replicates", str(1))
		self.settings.parser.set("general", "simphy_data_prefix",self.evolve[1])
		try:
			self.appLogger.info("Waiting for INDELible process to finish. This may take a while...")
			t = multiprocessing.Process(target=self.indelibleLauncher())
			t.start()
			t.join()
			self.appLogger.info("INDELible's run has finished.")
		except RuntimeError as verror:
			return   False, verror
		except Exception as ex:
			return   False, ex
		return True, "INDELible's run has finished."

	def indelibleLauncher(self):
		"""
		Generates a subprocess that handles the calling to INDELible
		"""
		indelibleMessage="INDELible run has finished";proc=""
		lines=[]
		try:
			cwd=os.getcwd()
			self.appLogger.info("Running INDELible")
			proc=""
			self.appLogger.info("Moving to {0}".format(self.settings.alignmentsFolderPath))
			os.chdir(self.settings.alignmentsFolderPath)
			self.appLogger.info("Running INDELible")
			proc = subprocess.check_output([self.settings.programCommand,'control.txt'],stderr=subprocess.STDOUT)
			if (sys.version_info>=(3,0)): proc=str(proc, "utf-8")
			self.appLogger.info("Moving back to working directory: {0}".format(cwd))
			os.chdir(cwd)
			cpuTime = [line.split(":")[1].split()[0] for line in proc.split('\n') if "* Block" in line]
			for item in range(1,len(cpuTime)):
				cpu=cpuTime[(item-1)]
				output="{0}_{1}".format(self.evolve[1],item )
				lines+=[item,cpu,output]
		except subprocess.CalledProcessError as error:
			indelibleMessage="\nINDELible execution error. "+\
			"\n------------------------------------------------------------------------"+\
			"\n{0}".format(error)+\
			"\n------------------------------------------------------------------------"+\
			"{0}".format(error.output)+\
			"\n------------------------------------------------------------------------"+\
			"\nFor more information about this error please run the following commands separately:\n"+\
			"\n\tcd {0}\n\tindelible\n".format(self.settings.alignmentsFolderPath)
			raise RuntimeError(indelibleMessage)
		except Exception as ex:
			exc_type, exc_obj, exc_tb = sys.exc_info()
			fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
			message="{1} | File: {2} - Line:{3}\n\t{0}".format(\
			    ex,exc_type,\
			    fname, exc_tb.tb_lineno\
			)
			raise RuntimeError(message)
		if (self.settings.runningTimes): self.writeRunningInfoIntoFile(lines)

	def writeRunningInfoIntoFile(self, lines):
		"""
		Writes the information about timing into a file.
		"""
		outputFile=os.path.join(
			self.settings.alignmentsFolderPath,\
			"{}.indelible.time".format(self.settings.projectName)
		)
		f=open(outputFile,"w")
		f.write("indexGT,cpuTime,outputFilePrefix\n")
		for item in lines:
			f.write(
				str(item[0])+","+\
				str(item[1])+","+\
				item[2]+"\n"
			)
		f.close()
		self.appLogger.info("File with timings of the INDELible run can be find on: {0}".format(outputFile))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import argparse,codecs,copy,csv,datetime,logging,multiprocessing,os,sys,threading,time,warnings
import numpy as np
import random as rnd
import settings as sp
from msatools import *
from coverage import NGSPhyDistribution as ngsphydistro

try:
	from collections import Counter
except ImportError:
	from counter import Counter

def getScoreSingle(data):
	"""
	Calulates a single Phred score.
	----------------------------------------------------------------------------
	Parameters:
	- data: the data that will be processed
	Returns:
	- the Phred score of the given value
	"""
	self.appLogger.debug("getScoreSingle(data)")
	if data!=0:
		return -10*np.log10(data)
	else:
		return 0
def getScoreMatrix(data):
	"""
	Calulates the Phred score of a matrix.
	----------------------------------------------------------------------------
	Parameters:
	- data: the data that will be processed
	Returns:
	- the Phred score of the given values
	"""
	self.appLogger.debug("getScoreMatrix(data)")
	value=np.copy(data)
	try:
		value=-10*np.log10(data)
	except:
		pass
	infs=np.inf==value
	value[infs]=0
	return value

class RunningInfo:
	"""
	Class for the storage of running time information Reads Counts Class
	Separated to be able to be used by several threads.
	"""
	def __init__(self, filename):
		self.filename=filename
		# self.appLogger=logging.getLogger('sngsw')
		self.lock = threading.Lock()
		self.lock.acquire()
		f=open(self.filename,"w")
		f.write("repID,locID,indID,inputFile,cpuTime,seed,outputFilePrefix\n")
		f.close()
		self.lock.release()


	def addLine(self, line):
		"""
		Adds a line to the running information estructure
		"""
		# self.appLogger.debug('Waiting for lock')
		self.lock.acquire()
		try:
			# self.appLogger.debug('Acquired lock')
			f=open(self.filename,"a")
			f.write(
				str(line[0])+","+\
				str(line[1])+","+\
				str(line[2])+","+\
				str(line[3])+","+\
				str(line[4])+","+\
				str(line[5])+","+\
				line[6]+"\n"
			)
			f.close()
		finally:
			# self.appLogger.debug('Released lock')
			self.lock.release()

class ReadCounts:
	"""
	Class to generate READ COUNTS of the given dataset
	----------------------------------------------------------------------------
	Atributes:
	- __NUCLEOTIDES: constant array with possible nucleotides characters
	- appLogger: logger to store status of the process flow
	- settings: Settings object withh all the program parameters
	- runningInfo: varible to store timing information of the processes.
	Structure to be shared across threads.
	- coverageFolderPath: path of the coverage matrix distribution.
	- settings.readsFolderPath: path where read count data will be stored.
	- readcountNoErrorFolderPath: path where true read count data will be stored.
	- readcountErrorFolderPath: path where sampled read count data will be
	stored.
	- numReplicates: number of species tree.
	- numLociPerReplicate: list with the number of loci per species tree
	- numIndividualsPerReplicate: list with the number of individuals per
	species tree.
	- numReplicatesDigits: number of digits that represent numReplicates
	- numLociPerReplicateDigits: list of the number of digits that represent
	numLociPerReplicate.
	- numIndividualsPerReplicateDigits: list of the number of digits that represent
	numIndividualsPerReplicate.
	- filteredReplicates: identifiers of the species tree replicates that will be used.
	"""

	__NUCLEOTIDES=["A","C","G","T"]
	appLogger=None
	settings=None
	runningInfo=None
	# path related variables
	readcountNoErrorFolderPath=""
	readcountErrorFolderPath=""

	numReplicates=None
	numLociPerReplicate=[]
	numIndividualsPerReplicate=[]
	numReplicatesDigits=[]
	numLociPerReplicateDigits=[]
	numIndividualsPerReplicateDigits=[]
	filteredReplicates=[]

	def __init__(self,settings):
		self.appLogger=logging.getLogger("ngsphy")
		self.appLogger.info('Read counts.')
		self.settings=settings

		self.numReplicates=self.settings.parser.getint("general", "numreplicates")
		cc=self.settings.parser.get("general", "numlociperreplicate").strip().split(",")
		self.numLociPerReplicate=[ int(item) for item in cc if not item ==""]
		cc=self.settings.parser.get("general", "numindividualsperreplicate").strip().split(",")
		self.numIndividualsPerReplicate=[ int(item) for item in cc if not item == ""]
		self.numReplicatesDigits=len(str(self.numReplicates))
		self.numLociPerReplicateDigits=[len(str(item)) for item in self.numLociPerReplicate]
		self.numIndividualsPerReplicateDigits=[len(str(item )) for item in self.numIndividualsPerReplicate]
		self.filteredReplicates=[int(item) for item in self.settings.parser.get("general", "filtered_replicates").strip().split(",")]
		if(self.settings.runningTimes):
			# Generating folder structure
			infoFile=os.path.join(\
				self.settings.outputFolderPath,\
				"{0}.info".format(self.settings.projectName)\
			)
			self.runningInfo=RunningInfo(infoFile)
			self.appLogger.info("File with timings of the [ngs-read-counts] per loci can be find on:\t{0}".format(infoFile))

		# naming variables for shorter codeline
		self.readcountNoErrorFolderPath=os.path.join(\
			self.settings.readsFolderPath,"no_error"\
		)
		self.readcountErrorFolderPath=os.path.join(\
			self.settings.readsFolderPath,"with_error"\
		)
		self.generateFolderStructureGeneral()

	def generateFolderStructureGeneral(self):
		"""
		Generates basic folder structure needed for Read Count option.
		This includes the folders:
		- reads
		- reads/true
		- reads/sampled
		- refereces
		"""
		self.appLogger.debug("generateFolderStructureGeneral(self)")
		self.appLogger.info("Creating folder structure for [ngs-read-counts]")
		try:
			os.makedirs(self.settings.readsFolderPath)
			self.appLogger.info("Generating output folder ({0})".format(\
				self.settings.readsFolderPath))
		except:
			self.appLogger.debug("Output folder exists ({0})".format(\
				self.settings.readsFolderPath))
		try:
			os.makedirs(self.readcountNoErrorFolderPath)
			self.appLogger.info("Generating output folder ({0})".format(\
				self.readcountNoErrorFolderPath))
		except:
			self.appLogger.debug("Output folder exists ({0})".format(\
				self.readcountNoErrorFolderPath))
		try:
			os.makedirs(self.readcountErrorFolderPath)
			self.appLogger.info("Generating output folder ({0})".format(\
				self.readcountErrorFolderPath))
		except:
			self.appLogger.debug("Output folder exists ({0})".format(\
				self.readcountErrorFolderPath))
		try:
			os.makedirs(self.settings.refAllelesFolderPath)
			self.appLogger.info("Generating output folder ({0})".format(\
				self.settings.refAllelesFolderPath))
		except:
			self.appLogger.debug("Output folder exists ({0})".format(\
				self.settings.refAllelesFolderPath))

	def generateFolderStructureDetail(self):
		"""
		Generates detailed folder structure needed for Read Count option.
		This includes:
		- Replicate folders  (as many as species tree replicates)
		- true/sampled folder per replicate.
		- These folders will contain the VCF files (true and sampled)
		"""
		self.appLogger.debug("generateFolderStructureDetail(self)")
		self.appLogger.info("Generating folders structure for read counts.")
		for i in range(0,len(self.filteredReplicates)):
			indexREP=self.filteredReplicates[i]
			self.appLogger.debug("{0}|{1}".format(i, self.filteredReplicates[i]))
			noErrorFolder=os.path.join(\
				self.readcountNoErrorFolderPath,\
				"REPLICATE_{0:0{1}d}".format(indexREP,self.numReplicatesDigits)\
			)
			errorFolder=os.path.join(\
				self.readcountErrorFolderPath,\
				"REPLICATE_{0:0{1}d}".format(indexREP,self.numReplicatesDigits)\
			)
			ref_alleles=os.path.join(\
				self.settings.refAllelesFolderPath,\
				"REPLICATE_{0:0{1}d}".format(indexREP,self.numReplicatesDigits)\
			)

			try:
				os.makedirs(noErrorFolder)
				self.appLogger.info("Generating output folder ({0})".format(noErrorFolder))
			except:
				self.appLogger.debug("Output folder exists ({0})".format(noErrorFolder))
			try:
				os.makedirs(errorFolder)
				self.appLogger.info("Generating output folder ({0})".format(errorFolder))
			except:
				self.appLogger.debug("Output folder exists ({0})".format(errorFolder))
			try:
				os.makedirs(ref_alleles)
				self.appLogger.info("Generating output folder ({0})".format(ref_alleles))
			except:
				self.appLogger.debug("Output folder exists ({0})".format(ref_alleles))


	def retrieveCoverageMatrix(self, indexREP):
		"""
		Reads coverage matrix for a specific species tree identifier
		------------------------------------------------------------------------
		Parameters:
		- indexREP:specific species tree identifier
		Returns:
		- coverage Matrix
		"""
		self.appLogger.debug("Retrieving coverage matrix")
		# coverage matrix per ST - row:indv - col:loci
		# each cov introduced as parameter is a NGSPhyDistribution
		coverageMatrixFilename=os.path.join(\
			self.settings.coverageFolderPath,\
			"{0}.{1:0{2}d}.coverage.csv".format(\
				self.settings.projectName,\
				indexREP,\
				self.numReplicatesDigits\
			)
		)
		coverageMatrix=np.zeros(\
			shape=(\
				self.numIndividualsPerReplicate[indexREP-1],\
				self.numLociPerReplicate[indexREP-1]\
			)\
		)
		firstLine=True; counter=0
		with open(coverageMatrixFilename, 'rb') as csvfile:
			coveragereader = csv.reader(csvfile, delimiter=',')
			for row in coveragereader:
				if firstLine:
					firstLine=False
				else:
					coverageMatrix[counter,]=[float(row[index]) for index in range(1,len(row))]
					counter+=1
				if not counter < self.numIndividualsPerReplicate[indexREP-1]: break
		return coverageMatrix


	def parseReferenceAllelesList(self, filename):
		"""
		Used to parse referenceList, file with format: STID,SPID,TIPID
		-----------------------------------------------------------------------
		Parameters:
		- filename: path of the referenceList file.
		There's only ONE file with the relation of the ref_alleles
		If "None" inputted (file is missing) then reference by default is 1_0_0
		for all species tree replicates.
		Returns:
		- output: list. each element of the list is a triplet
				(REPLICATEID,SPID,LOCID,GENEID)
		"""
		self.appLogger.debug("parseReferenceAllelesList(self, filename)")
		self.appLogger.info("Retrieving identifiers of the reference alleles...")
		referenceList=[]
		if filename:
			# There's a file
			filepath=os.path.abspath(filename)
			lines=None
			with open(filepath) as f:
				lines=[line.strip().split(",") for line in f]
			lines=sorted(lines, key=lambda x:x[1])
			skipped=False
			message="Parsing reference list. "+\
				"A default reference has been introduced.\n"+\
				"Replicate index:"

			index=0; itemIndex=0
			while (index < len(lines)) and (itemIndex < self.numReplicates):
				d=lines[index]
				if  (int(d[0]) == (itemIndex+1)) and (not "" in d):
					try:
						referenceList+=[(d[0],d[1],d[2],d[3])]
					except IndexError as ie:
						skipped=True
					index+=1
				else: skipped=True

				if skipped:
					self.appLogger.warning("{0}{1}".format(message,(index+1)))
					referenceList+=[(index+1,1,0,0)]
					skipped=False
				itemIndex+=1

			if (itemIndex < self.numReplicates):
			    for i in range(self.numReplicates):
			        print([i+1,1,0,0])
		else:
			# iterate get a list with same decription for all species trees
			for item in range(0,self.numReplicates):
				referenceList+=[(item+1,1,0,0)]
		return referenceList

	def extractTrueVariantsPositions(self, filename):
		"""
		Extract true variant positions from the MSA file used as input
		------------------------------------------------------------------------
		Parameters:
		- filename: MSA fasta file from where to extract the variable positions
		Returns:
		- dictionary. indices=variable sites, content=variable nucleotide set
		"""
		self.appLogger.debug("extractTrueVariantsPositions(self, filename)")
		filepath=os.path.abspath(filename)
		lines=[];variants=dict();seqDescriptions=[]
		numTotalSeqs=0;lenSeq=0; matrix=None
		self.appLogger.debug(\
			"Extracting variable positions from: {0}".format(\
			filepath
		))
		# Checking sequence length
		if isFasta(filepath):
			seq=None
			with open(filepath,"r") as f:
				f.readline()
				seq=f.readline().strip()
			lenSeqs=len(seq.strip())
		else:
			message="{0}\n{1}\n{2}\n".format(\
				"File has a wrong file format.",\
				filepath,\
				"Please check. Exiting."
			)
			raise TypeError(message)
			# If I get here, function run is over - goes back to  run()

		with open(filepath,"r") as f:
			for line in f:
				if line.startswith(">"): numTotalSeqs+=1
			matrix=np.chararray((numTotalSeqs,lenSeqs), itemsize=1)
		for i in range(0, numTotalSeqs):
			for j in range(0,lenSeqs):
				matrix[i,j]=""

		lines=None
		with open(filepath,"r") as f:
			lines=[line.strip() for line in f if not line.strip()==""]

		numLinesFile=len(lines);index=0
		indexSeqs=range(1,numLinesFile,2)
		for i in range(0,numTotalSeqs):
			seqDescriptions+=[lines[indexSeqs[i]-1]]
			for j in range(0, lenSeqs):
				matrix[i,j]=lines[indexSeqs[i]][j]

		for indexCol in range(0,matrix.shape[1]):
			c=Counter(matrix[:,indexCol])
			l=np.unique(matrix[:,indexCol])
			# sys.stdout.write("Pos <{0}>\n{1}|{2}\n".format(indexCol, ",".join(c.keys()), ",".join([str(d) for d in c.values()])))
			if (len(l)>1):
				variants[str(indexCol)]=c.keys()
		return variants

	def parseIndividualRelationFile(self,filename):
		"""
		Parses "Individual-description relation" file per species tree
		in order to obtain information on how sequences from INDELible
		are related. Generated within NGSPhy.
		------------------------------------------------------------------------
		Parameters:
		- filename: path for the Individual-description relation file
		Returns:
		- if ploidy=1:	returns a dict. key: indID,content: dict(repID,spID,locID, geneID)
		- if ploidy=2:	returns a dict. key: indID,content: dict(repID,spID,locID, mateID1,mateID2)
		"""
		self.appLogger.debug("parseIndividualRelationFile(self,filename)")
		individuals=dict()
		if (self.settings.ploidy>0 and self.settings.ploidy<=2):
			with open(os.path.abspath(filename)) as csvfile:
				d = csv.DictReader(csvfile)
				if (self.settings.ploidy==1):
					for row in d:
						individuals[str(row["indID"])] = {\
							"repID":row["repID"],\
							"spID":row["spID"],\
							"locID":row["locID"],\
							"geneID":row["geneID"]}
				if (self.settings.ploidy==2):
					# repID,indID,spID,mateID1,mateID2
					for row in d:
						individuals[str(row["indID"])] = {\
							"repID":row["repID"],\
							"spID":row["spID"],\
							"locID":row["locID"],\
							"mateID1":row["mateID1"],\
							"mateID2":row["mateID2"]}
				csvfile.close()
		else:
			# There has been a verification in Settings class, but just in case.
			message="\n\t{0}\n\t{1}".format(\
				"Ploidy assigned is incorrect.",\
				"Please verify. Exciting."
			)
			raise ValueError(message)
		return individuals

	def getDepthCoveragePerIndividual(self, numVarSites,startingCoverage):
		"""
		Compute coverage per individuals
		since the value for SNPS is fixed to a Negative Binomial distribution
		I iterate over the number of variants for this specific locus and
		since everythime i hit play! (called value()) I'll sample a new value
		from the previously set distribution, i'll then get a random value from a
		negative bionmial distribution with mean max-coverage
		------------------------------------------------------------------------
		Parameters:
		- numVariableSites: number of variable sites
		- startingCoverage: starting value to calculate coverage
		Returns:
		- list of values that correspond to the coverage per each snp of the MSA
		"""
		self.appLogger.debug(\
			"{} - ({},{})".format(\
				"getDepthCoveragePerIndividual(self, numVarSites,startingCoverage)",\
				numVarSites,\
				startingCoverage\
			)\
		)
		distro=ngsphydistro("nb",[startingCoverage,startingCoverage])
		DP=distro.value(numVarSites)
		return DP

	def getHaploidIndividualSequence(self,msa,ind):
		"""
		Extract individuals "ind" sequence(s) from MSA dictionary
		------------------------------------------------------------------------
		Parameters:
		- msa: dictionary
		- ind: dict(indexREP,seqDEscription)
		Returns:
		- sequence of the individual
		"""
		# ind: [indID,indexREP,seqDescription]
		self.appLogger.debug("getHaploidIndividualSequence(self,msa,ind)")
		seqSize=len(msa["{0}_{1}".format(str(1),str(0))][str(0)]['sequence'])
		fullInd=None; speciesID=None; tipID=None; tmp=None
		fullInd=np.chararray(shape=(1,seqSize), itemsize=1)
		speciesID=ind["spID"].strip()
		locusID=ind["locID"].strip()
		tipID=ind["geneID"].strip()
		tmp=list(msa["{0}_{1}".format(str(speciesID), str(locusID))][str(tipID)]['sequence'])
		fullInd=[item for item in tmp]
		return fullInd

	def getDiploidIndividualSequence(self,msa,ind):
		"""
		Extract individuals "ind" sequence(s) from MSA dictionary
		Parameters:
		- msa: dictionary
		- ind: dict(repID,spID, mateID1,mateID2)
		Returns:
		- matrix: 2 x seqLength - representing the sequence of the individual
		"""
		# ind:[indID, indexREP,speciesID, mateID1,mateID2]
		self.appLogger.debug("getDiploidIndividualSequence(self,msa,ind)")
		seqSize=len(msa["1_0"][str(0)]['sequence'])
		fullInd=None;  tmp=None
		speciesID=ind["spID"];
		locusID=ind["locID"];
		tipID1=ind["mateID1"];
		tipID2=ind["mateID2"];
		geneFamily="{0}_{1}".format(speciesID,locusID)
		fullInd=np.chararray(shape=(2,seqSize), itemsize=1)
		for i in range(0,seqSize):
			fullInd[0,i]="";fullInd[1,i]=""
		tmp1=list(msa[geneFamily][str(tipID1)]['sequence'])
		tmp2=list(msa[geneFamily][str(tipID2)]['sequence'])
		for i in range(0,seqSize):
			fullInd[0,i]=tmp1[i]
			fullInd[1,i]=tmp2[i]
		return fullInd


	def computeHaploid(self,indexREP,indexLOC,msa,individuals,refAllelesFilePath,\
		referenceDescription,referenceSeqFull,variableSites,DP):
		"""
		compute the READ COUNT for the specic ploidy
		------------------------------------------------------------------------
		Parameters:
		- indexREP: species tree id
		- indexLOC: locus/gene tree id
		- msa: parsed multiple sequence alignent file (dictionary)
		- individuals: parsed individual-description relation file (dictionary)
		- refAllelesFilePath: where the reference is written.
		- referenceSeqFull: reference sequence as is
		- variableSites: dictionary. keys: variable positions. content: variable nucleotide set
		- DP: variable sites coverage
		Returns:
		- get a file written in the STreplicate folder (true/sampled)
		"""
		self.appLogger.debug("computeHaploid(...)")
		nInds=len(individuals)
		nVarSites=len(variableSites.keys())
		# Getting the proper indices of the variable sites
		variableSitesPositionIndices=np.sort([int(pos) for pos in variableSites.keys()])
		alt=dict();  altInds=dict()
		rcsNOError=dict();rcsWErrors=dict()
		HTGeneral=dict();HLGeneral=dict();ADGeneral=dict()
		HTGeneralWErrors=dict();HLGeneralWErrors=dict();ADGeneralWErrors=dict()
		for pos in variableSites.keys():
			alt[pos]=[]; altInds[pos]=[]
		for pos in variableSitesPositionIndices:
			# sys.stdout.write('POS <{0}>: set 1 - {1} | set 2 - {2}\n'.format(pos,variableSites[str(pos)],[referenceSeqFull[pos]]) )
			tmpset=set(variableSites[str(pos)])-set([referenceSeqFull[pos]])
			if tmpset== set():
				alt[str(pos)]=[]
			else:
				alt[str(pos)]=list(tmpset)
		for index in range(0,nInds):
			indexIND=str(index)
			HTGeneral[indexIND]=[];HLGeneral[indexIND]=[];ADGeneral[indexIND]=[]
			HTGeneralWErrors[indexIND]=[];
			HLGeneralWErrors[indexIND]=[];ADGeneralWErrors[indexIND]=[]
			rcsNOError[indexIND]=[];rcsWErrors[indexIND]=[]
		########################################################
		# TRUE
		########################################################
		# iterate over individuals

		self.appLogger.debug("NOError")
		altInd=copy.copy(alt)
		for indIndex in range(0,nInds):
			indexIND=str(indIndex)
			self.appLogger.debug(\
				"\tReplicate tree: {0} - Locus {1} |  Individual {2} ({3}/{4})".format(\
					indexREP,\
					indexLOC,\
					indIndex,\
					(indIndex+1),\
					nInds)\
				)
			ind=individuals[indexIND]
			individualSeq=self.getHaploidIndividualSequence(msa,ind)
			# individualSeq[202]+="**"
			# AD per individual different if true/sampledInd
			ADNOErrors,ADWErrors=self.getAllelicDepthPerHaploidIndividual(\
				individualSeq,variableSitesPositionIndices,DP[indIndex])
			rcNOError,rcWErrors=self.getReadCountPerIndividual(\
				ADNOErrors,ADWErrors,variableSitesPositionIndices)
			altInd=self.getAltUpdatedPerIndividual(\
				referenceSeqFull,altInd,ADWErrors)
			HTNOError=self.gettingHaplotype(\
				referenceSeqFull,individualSeq,alt,variableSitesPositionIndices)
			HLNOError=self.haplotypeLikehood(\
				rcNOError,variableSitesPositionIndices,0)
			rcsNOError[indexIND]=rcNOError
			rcsWErrors[indexIND]=rcWErrors
			ADGeneral[indexIND]=ADNOErrors
			ADGeneralWErrors[indexIND]=ADWErrors
			HTGeneral[indexIND]=HTNOError
			HLGeneral[indexIND]=HLNOError

		# here corresponds to the INDEXST that is going to be written
		self.writeVCFFile(\
			indexREP,indexLOC,\
			refAllelesFilePath,referenceDescription, referenceSeqFull,\
			alt,variableSitesPositionIndices,\
			HTGeneral,HLGeneral,\
			ADGeneral,DP,True)

		########################################################
		# With errors
		########################################################
		self.appLogger.debug("With errors")
		for indIndex in range(0,nInds):
			indexIND=str(indIndex)
			self.appLogger.info("Individual {0} ({1}/{2})".format(indIndex,(indIndex+1),nInds))
			ind=individuals[indexIND]
			individualSeq=self.getHaploidIndividualSequence(msa,ind)
			HTGeneralWErrors[indexIND]=self.gettingHaplotype(\
				referenceSeqFull,individualSeq,\
				altInd, variableSitesPositionIndices)
			HLGeneralWErrors[indexIND]=self.haplotypeLikehood(\
				rcsWErrors[indexIND],\
				variableSitesPositionIndices,\
				self.settings.readCountsError)

		self.writeVCFFile(
			indexREP,indexLOC,\
			refAllelesFilePath,referenceDescription, referenceSeqFull,\
			altInd,variableSitesPositionIndices,\
			HTGeneralWErrors,HLGeneralWErrors,\
			ADGeneralWErrors,DP,False)

	def haplotypeLikehood(self,variantsRC,variableSitesPositionIndices,error):
		"""
		computes log10-scaled haplotype likelihood per variable site
		------------------------------------------------------------------------
		Parameters:
		- variantsRC: read count pairs of the variable positions
		- variableSitesPositionIndices: indices of the variable sites position
		- error: sequence error
		Returns:
		- GL: dictionary.
		- keys: positons.
		- content: log10-scaled likelihood for the specific variable site
		"""
		self.appLogger.debug("haplotypeLikehood(self,variantsRC,observed,variableSites,error)")
		nVariants=len(variableSitesPositionIndices)
		HL=np.ones(shape=(4,nVariants), dtype=np.float)
		error=float(error)
		for indexVar in range(0,nVariants):
			indexReads=str(variableSitesPositionIndices[indexVar])
			reads=variantsRC[indexReads]
			# possible haplotypes
			for indexNuc in range(0,4):
				nuc=self.__NUCLEOTIDES[indexNuc]
				# read to be analyzed
				for b in reads:
					if (b.upper()==nuc.upper()):
						HL[indexNuc,indexVar]=HL[indexNuc,indexVar]*(1-error)
					else:
						HL[indexNuc,indexVar]=HL[indexNuc,indexVar]*(error/3)
				# print indexVar, "b: ",b, " A:",nuc,",".join(HL[:,indexVar].astype(str))
		value=np.copy(HL)
		with warnings.catch_warnings():
			warnings.filterwarnings('error')
			try:
				value=np.log10(HL)
			except:
				pass
			infs=np.inf==value
			value[infs]=-300
		return value

	def gettingHaplotype(self,ref,seq,alt, variableSitesPositionIndices):
		"""
		get haplotype for the given ref-seq-alt triplet
		------------------------------------------------------------------------
		Parameters:
		- ref: reference sequence
		- seq: individual sequnece
		- alt: dictionary with alternative alleles per variable position
		- variableSitesPositionIndices: variableSites
		Returns:
		- GT: dictionary. keys: positons. content: haplotype for the specific variable site
		"""
		self.appLogger.debug("gettingHaplotype(self,ref,seq,alt, variableSitesPositionIndices)")
		GT=dict()
		# init variantsRC
		for indexVar in variableSitesPositionIndices:
			GT[str(indexVar)]=0

		for indexVar in variableSitesPositionIndices:
			refPos=ref[indexVar]
			seqPos=seq[indexVar]
			altValues=alt[str(indexVar)]
			iv=str(indexVar)
			for index in range(0,len(altValues)):
				altToCompare=altValues[index]
				if (index==0) and (altToCompare==seqPos):
					GT[iv]=1
				if (index==1) and (altToCompare==seqPos):
					GT[iv]=2
				if (index==2) and (altToCompare==seqPos):
					GT[iv]=3
		return copy.copy(GT)

	def getAltUpdatedPerIndividual(self,ref,alt,AD):
		"""
		update general alternative alleles list
		------------------------------------------------------------------------
		Parameters:
		- referenceSeq: reference sequence
		- alt: dictionary. current alternative allele list per variant
		- AD: allelic depth matrix
		Return:
		- newAlt: dictionary. keys: positions. content: alt alleles corresponding to that position
		"""
		self.appLogger.debug("getAltUpdatedPerIndividual(self,ref,alt,AD)")
		# update alt values
		# With errors - ALT is a DICT
		# altUpdate is going to be a dict too
		altUpdated=dict()
		for item in alt.keys(): altUpdated[item]=[]
		sortedAltKeys=np.sort([int(item) for item in alt.keys()])
		for index in range(0,len(sortedAltKeys)):
			possibleNucsAlt=[]
			if (AD[0,index]>0): possibleNucsAlt+=["A"]
			if (AD[1,index]>0): possibleNucsAlt+=["C"]
			if (AD[2,index]>0): possibleNucsAlt+=["G"]
			if (AD[3,index]>0): possibleNucsAlt+=["T"]
			newAlt=set(\
					alt[str(sortedAltKeys[index])] + \
					possibleNucsAlt\
				) - \
				set(ref[sortedAltKeys[index]])
			altUpdated[str(sortedAltKeys[index])]+=list(np.sort(list(newAlt)))
		return altUpdated

	def getReadCountPerIndividual(self,ADNOErrors,ADWErrors, variableSitesPositionIndices):
		"""
		generates list of read counts per variant per individual
		------------------------------------------------------------------------
		Parameters:
		- ADNOErrors: NOError allelic depth
		- ADWErrors: With errors allelic depth
		- variableSitesPositionIndices: position of the variable sites
		Return:
		- RCNOError, RCWErrors.
		"""
		self.appLogger.debug("getReadCountPerIndividual(self,ADNOErrors,ADWErrors, variableSitesPositionIndices)")
		nVariants=len(variableSitesPositionIndices)
		rcNOError=dict();rcWErrors=dict()
		# init variantsRC
		for indexVar in range(0,nVariants):
			indexConverted=str(variableSitesPositionIndices[indexVar])
			rcNOError[indexConverted]=[]
			rcWErrors[indexConverted]=[]
		# getting read count
		for indexVar in range(0,nVariants):
			for indexNuc in range(0,4):
				nuc=[self.__NUCLEOTIDES[indexNuc]]
				indexConverted=str(variableSitesPositionIndices[indexVar])
				val=ADNOErrors[indexNuc,indexVar]
				rcNOError[indexConverted]+=nuc*val
				val=ADWErrors[indexNuc,indexVar]
				rcWErrors[indexConverted]+=nuc*val
		return rcNOError, rcWErrors

	def getAllelicDepthPerHaploidIndividual(self,individualSeq,variableSitesPositionIndices,DP):
		"""
		Generate allelic depth per individual per site
		------------------------------------------------------------------------
		Parameters:
		- individualSequence: sequence of the individual
		- variableSitesPositionIndices: position of the variable sites
		- DP: coverage of the variable sites
		Returns:
		- ADNOErrors, ADWErrors.
		"""
		self.appLogger.debug("getAllelicDepthPerHaploidIndividual(self,fullInd,variableSites,DP)")
		nVariants=len(variableSitesPositionIndices)
		ADNOErrors=np.zeros(shape=(4,nVariants), dtype=np.int)
		ADWErrors=np.zeros(shape=(4,nVariants), dtype=np.int)
		self.appLogger.debug("Starting to iterate over the variants")
		# There was a problem in this block related to low coverage, error
		# and possible substitutions.
		for indexVar in range(0,nVariants):
			finalRC=[];indNucs=[]
			# getting general coverage per position
			posCoverage=DP[indexVar]
			indexSeq=variableSitesPositionIndices[indexVar]
			indNucs=individualSeq[indexSeq]
			finalRC=[indNucs]*(posCoverage)
			counter=Counter(finalRC)
			# TRUE READ count
			ADNOErrors[0,indexVar]=counter["A"];ADNOErrors[1,indexVar]=counter["C"]
			ADNOErrors[2,indexVar]=counter["G"];ADNOErrors[3,indexVar]=counter["T"]
			# SAMPLE READ COUNT - need to know error distribution
			errorDistro=ngsphydistro("b",[posCoverage,self.settings.readCountsError])
			errorD=errorDistro.value(1)[0]
			errorPositions=[]
			# need to know possible nucleotides to substitute my position with error
			possibleNucs=list(set(self.__NUCLEOTIDES)-set([indNucs]))
			# I have some positions (at least 1) that is an error
			# errorD= array with coded error nucleotides that will be modified
			if (errorD>0):
				errorChoices=np.random.choice(possibleNucs, int(errorD), replace=True)
				maxAvailablePositions=posCoverage
				if not ((posCoverage % 2) == 0): maxAvailablePositions=posCoverage-1
				if maxAvailablePositions == 0:  maxAvailablePositions=posCoverage

				errorPositions=np.random.randint(maxAvailablePositions,size=int(errorD))
				for item in range(0,len(errorPositions)):
					posToChange=errorPositions[item]
					finalRC[posToChange]=errorChoices[item]
			counter=Counter(finalRC)
			# if any of the nucleotides does not have a counter retrieving it will be 0
			ADWErrors[0,indexVar]=counter["A"];ADWErrors[1,indexVar]=counter["C"]
			ADWErrors[2,indexVar]=counter["G"];ADWErrors[3,indexVar]=counter["T"]
		return ADNOErrors,ADWErrors

	def computeDiploid(self,indexREP,indexLOC,msa,individuals,refAllelesFilePath,\
		referenceDescription,referenceSeqFull,variableSites,DP):
		"""
		compute the READ COUNT for the specic ploidy
		------------------------------------------------------------------------
		Parameters:
		- indexREP: species tree id
		- indexLOC: locus/gene tree id
		- msa: parsed multiple sequence alignent file (dictionary)
		- individuals: parsed individual-description relation file (dictionary)
		- referenceSeqFull: reference sequence as is
		- variableSites: dictionary. keys: variable positions. content: variable nucleotide set
		- DP: variable sites coverage
		Returns:
		- get a file written in the STreplicate folder (true/sampled)
		"""
		self.appLogger.debug("computeDiploid(...)")
		nInds=len(individuals)
		nVarSites=len(variableSites.keys())
		# Getting the proper indices of the variable sites
		variableSitesPositionIndices=np.sort([int(pos) for pos in variableSites.keys()])
		alt=dict();  altInds=dict()
		rcsNOError=dict();rcsWErrors=dict()
		GTGeneral=dict();GLGeneral=dict();ADGeneral=dict()
		GTGeneralWErrors=dict();GLGeneralWErrors=dict();ADGeneralWErrors=dict()
		for pos in variableSites.keys():
			alt[pos]=[]; altInds[pos]=[]
		for pos in variableSitesPositionIndices:
			alt[str(pos)]=list(set(variableSites[str(pos)])-set([referenceSeqFull[pos]]))
		for index in range(0,nInds):
			indexIND=str(index)
			GTGeneral[indexIND]=[];GLGeneral[indexIND]=[];ADGeneral[indexIND]=[]
			GTGeneralWErrors[indexIND]=[];
			GLGeneralWErrors[indexIND]=[];ADGeneralWErrors[indexIND]=[]
			rcsNOError[indexIND]=[];rcsWErrors[indexIND]=[]
		########################################################
		# DIPLOID TRUE
		########################################################
		# iterate over individuals
		self.appLogger.debug("NOError")
		altInd=copy.copy(alt)
		for indIndex in range(0,nInds):
			GTNOError=dict()
			indexIND=str(indIndex)
			self.appLogger.debug("\tSpecies tree: {0} - Locus {1} |  Individual {2} ({3}/{4})".format(\
				indexREP, indexLOC,
				indIndex,(indIndex+1),nInds))
			ind=individuals[indexIND]
			individualSeq=self.getDiploidIndividualSequence(msa,ind)
			ADNOErrors,ADWErrors=self.getAllelicDepthPerDiploidIndividual(\
				individualSeq,variableSitesPositionIndices,DP[indIndex])
			rcNOError,rcWErrors=self.getReadCountPerIndividual(\
				ADNOErrors,ADWErrors,variableSitesPositionIndices)
			altInd=self.getAltUpdatedPerIndividual(\
				referenceSeqFull,altInd,ADWErrors)
			GTNOErrorS0=self.gettingHaplotype(
				referenceSeqFull,individualSeq[0,:],\
				alt, variableSitesPositionIndices
			)
			GTNOErrorS1=self.gettingHaplotype(
				referenceSeqFull,individualSeq[1,:],\
				alt, variableSitesPositionIndices
			)
			for item in variableSitesPositionIndices:
				GTNOError[str(item)]=[GTNOErrorS0[str(item)],GTNOErrorS1[str(item)]]
			GLNOError=self.genotypeLikehood(\
				rcNOError,variableSitesPositionIndices,0)
			rcsNOError[indexIND]=rcNOError
			rcsWErrors[indexIND]=rcWErrors
			ADGeneral[indexIND]=ADNOErrors
			ADGeneralWErrors[indexIND]=ADWErrors
			GTGeneral[indexIND]=GTNOError
			GLGeneral[indexIND]=GLNOError

		# here corresponds to the INDEXST that is going to be written
		self.writeVCFFile(\
			indexREP,indexLOC,\
			refAllelesFilePath,referenceDescription, referenceSeqFull,\
			alt,variableSitesPositionIndices,\
			GTGeneral,GLGeneral,\
			ADGeneral,DP,True)

		########################################################
		# DIPLOID SAMPLED
		########################################################
		self.appLogger.debug("With errors")
		for indIndex in range(0,nInds):
			GTWErrors=dict()
			indexIND=str(indIndex)
			self.appLogger.debug("\tSpecies tree: {0} - Locus {1} |  Individual {2} ({3}/{4})".format(\
				indexREP, indexLOC,
				indIndex,(indIndex+1),nInds))
			ind=individuals[indexIND]
			individualSeq=self.getDiploidIndividualSequence(msa,ind)
			GTWErrorsS0=self.gettingHaplotype(
				referenceSeqFull,individualSeq[0,:],\
				altInd, variableSitesPositionIndices
			)
			GTWErrorsS1=self.gettingHaplotype(
				referenceSeqFull,individualSeq[1,:],\
				altInd, variableSitesPositionIndices
			)
			for item in variableSitesPositionIndices:
				GTWErrors[str(item)]=[GTWErrorsS0[str(item)],GTWErrorsS1[str(item)]]
			GLWErrors=self.genotypeLikehood(\
				rcsWErrors[indexIND],\
				variableSitesPositionIndices,\
				self.settings.readCountsError)
			GTGeneralWErrors[indexIND]=GTWErrors
			GLGeneralWErrors[indexIND]=GLWErrors

		self.writeVCFFile(
			indexREP,indexLOC,\
			refAllelesFilePath,referenceDescription,referenceSeqFull,\
			altInd,variableSitesPositionIndices,\
			GTGeneralWErrors,GLGeneralWErrors,\
			ADGeneralWErrors,DP,False)

	def genotypeLikehood(self, variantsRC,variableSitesPositionIndices,error):
		"""
		compute the genotype likelihoods of a specific ngs-read-counts set
		------------------------------------------------------------------------
		Parameters:
		- variantsRC:
		- variableSitesPositionIndices:
		- error
		Returns:
		- Genotype likelihoods for the given variable positions and error
		"""
		self.appLogger.debug("genotypeLikehood(self, variantsRC,variableSitesPositionIndices,error)")
		nVariants=len(variableSitesPositionIndices)
		GL=dict();GLout=dict() # Initialization of genotypeLikehood variable
		possibleGenotypes=self.getAllPossibleGenotypes()
		for indexVar in variableSitesPositionIndices:
			GL[str(indexVar)]={}
			GLout[str(indexVar)]={}
			for pg in possibleGenotypes:
				gen="".join(pg)
				GL[str(indexVar)][gen]=1
				GLout[str(indexVar)][gen]=1
		# getting likelihoods

		for indexVar in variableSitesPositionIndices:
			for pg in possibleGenotypes:
				gen="".join(pg)
				reads=variantsRC[str(indexVar)]
				for b in reads:
					A1=pg[0];A2=pg[1]
					valAl1=0; valA2=0
					if (b==A1):
						valA1=0.5*(1-error)
					else:
						valA1=0.5*(error/3)
					if (b==A2):
						valA2=0.5*(1-error)
					else:
						valA2=0.5*(error/3)
					GL[str(indexVar)][gen]=\
						GL[str(indexVar)][gen]*(valA1+valA2)
		for indexVar in variableSitesPositionIndices:
			for pg in possibleGenotypes:
				gen="".join(pg)
				value=GL[str(indexVar)][gen]
				if value==0:
					GLout[str(indexVar)][gen]=-300
				else:
					GLout[str(indexVar)][gen]=np.log10(value)
		return GL

	def getAllPossibleGenotypes(self):
		"""
		Generates all the possible genotypes
		------------------------------------------------------------------------
		returns a dictionary. key: positions. content [A1,A2]
		"""
		self.appLogger.debug("getAllPossibleGenotypes(self)")
		possibleGenotypes=[]
		for pos1 in self.__NUCLEOTIDES:
			for pos2 in self.__NUCLEOTIDES:
				possibleGenotypes+=[[pos1,pos2]]
		return possibleGenotypes

	def getPossibleGenotypesPerVariableSite(self,ref,alt, variableSitesPositionIndices):
		"""
		get the genotypes for a specfic position
		------------------------------------------------------------------------
		intput:
			ref
			alt
			variableSitesPositionIndices
		output:
			returns a dictionary. key: positions. content [A1,A2]
		"""
		self.appLogger.debug("getPossibleGenotypesPerVariableSite(self,ref,alt, variableSites)")
		self.appLogger.debug("Getting genotypes")
		possibleGenotypes=dict();elems=dict()
		# Initialization
		for varSite in variableSitesPositionIndices:
			possibleGenotypes[str(varSite)]=[]
			elems[str(varSite)]=[]

		for varSite in variableSitesPositionIndices:
			elems[str(varSite)]=[ref[varSite]]+alt[str(varSite)]

		for indexAlt in elems.keys():
			altPos=elems[indexAlt]
			d=[]
			for pos1 in altPos:
				for pos2 in altPos:
					if not (set([pos1,pos2]) in d):
						d+=[set([pos1,pos2])]
						possibleGenotypes[str(indexAlt)]+=["".join([pos1,pos2])]

		return possibleGenotypes

	def genotypeOrder(self,alleles):
		ordered=[]
		for a in range(0,len(alleles)):
			for b in range(0,(a+1)):
				ordered+=[[alleles[a],alleles[b]]]
		return ordered


	def getAllelicDepthPerDiploidIndividual(self,individualSeq,variableSitesPositionIndices,DP):
		"""
		Generate allelic depth per individual per site
		------------------------------------------------------------------------
		intput:
			individualSequence: sequence of the individual
			variableSitesPositionIndices: position of the variable sites
			DP: coverage of the variable sites
		output:
			ADNOErrors, ADWErrors.
		"""
		self.appLogger.debug("getAllelicDepthPerDiploidIndividual(self,fullInd,variableSitesPositionIndices,DP)")
		nVariants=len(variableSitesPositionIndices)
		ADNOErrors=np.zeros(shape=(4,nVariants), dtype=np.int)
		ADWErrors=np.zeros(shape=(4,nVariants), dtype=np.int)
		self.appLogger.debug("Starting to iterate over the variants")
		# There was a problem in this block related to low coverage, error
		# and possible substitutions.
		for indexVar in range(0,nVariants):
			finalRC=[];indNucs=[]
			# getting general coverage per position
			posCoverage=DP[indexVar]
			indexSeq=variableSitesPositionIndices[indexVar]
			indNucStrand1=individualSeq[0,indexSeq]
			indNucStrand2=individualSeq[1,indexSeq]
			diploidDistro=ngsphydistro("b",[posCoverage,0.5])
			diploidCoverage=diploidDistro.value(1)[0]
			finalRC=[indNucStrand1]*(diploidCoverage)
			finalRC+=[indNucStrand2]*(posCoverage-diploidCoverage)

			counter=Counter(finalRC)
			# TRUE READ count
			ADNOErrors[0,indexVar]=counter["A"];ADNOErrors[1,indexVar]=counter["C"]
			ADNOErrors[2,indexVar]=counter["G"];ADNOErrors[3,indexVar]=counter["T"]
			# SAMPLE READ COUNT - need to know error distribution
			errorDistro=ngsphydistro("b",[posCoverage,self.settings.readCountsError])
			errorD=errorDistro.value(1)[0]

			errorPositions=[]
			# need to know possible nucleotides to substitute my position with error
			possibleNucs=list(set(self.__NUCLEOTIDES)-set([indNucStrand1]+[indNucStrand2]))
			# I have some positions (at least 1) that is an error
			# errorD= array with coded error nucleotides that will be modified
			if (errorD>0):
				errorChoices=np.random.choice(possibleNucs, int(errorD), replace=NOError)
				maxAvailablePositions=posCoverage
				if not ((posCoverage % 2) == 0): maxAvailablePositions=posCoverage-1
				if maxAvailablePositions == 0:  maxAvailablePositions=posCoverage

				errorPositions=np.random.randint(maxAvailablePositions,size=int(errorD))
				for item in range(0,len(errorPositions)):
					posToChange=errorPositions[item]
					finalRC[posToChange]=errorChoices[item]
				counter=Counter(finalRC)
			# if any of the nucleotides does not have a counter retrieving it will be 0
			ADWErrors[0,indexVar]=counter["A"];ADWErrors[1,indexVar]=counter["C"]
			ADWErrors[2,indexVar]=counter["G"];ADWErrors[3,indexVar]=counter["T"]
		return ADNOErrors,ADWErrors


	def formatIndividualDataForVCF(self,ref,alt,variableSitesPositionIndices,HT,HL,AD,DP):
		"""
		gets single individual data formated as a GT:GL:AD:DP VCF column
		------------------------------------------------------------------------
		intput:
			reference sequence
			alternative alleles
			variableSitesPositionIndices
			haplotype
			likelihood
			allelic depth
			read depth
		output:
			string column with information in format: GT:GL:AD:DP
		"""
		self.appLogger.debug("formatIndividualDataForVCF(self,ref,alt,variableSitesPositionIndices,HT,HL,AD,DP)")
		nVariants=len(variableSitesPositionIndices)
		nInds=len(HT.keys())
		allVariants=dict()
		possibleGenotypes=None
		if self.settings.ploidy==2:
			possibleGenotypes=self.getPossibleGenotypesPerVariableSite(ref,alt,variableSitesPositionIndices)

		for indexVAR in variableSitesPositionIndices:
			allVariants[str(indexVAR)]=[]
		# Had an error here, because the alt variable was empty (passed wrong variable name to the function)
		for indexVAR in range(0,nVariants):
			tmpInd=variableSitesPositionIndices[indexVAR]
			for indexIND in range(0,nInds):
				trueRows=None; htPerInd=None; adPerInd=None;hlPerInd=None;
				if self.settings.ploidy==1:
					valuesToCodify=[ref[tmpInd]]+alt[str(tmpInd)]
					trueRows=self.codifySequences(valuesToCodify)
					htPerInd="{}".format(HT[str(indexIND)][str(tmpInd)])
					hlPerInd=",".join(HL[str(indexIND)][trueRows,indexVAR].astype(dtype=int).astype(dtype=np.str))
					adPerInd=",".join(AD[str(indexIND)][trueRows,indexVAR].astype(dtype=int).astype(dtype=np.str))
				if self.settings.ploidy==2:
					valuesToCodify=[ref[tmpInd]]+alt[str(tmpInd)]
					trueRows=["".join(pair) for pair in self.genotypeOrder(valuesToCodify)]
					htVar=HT[str(indexIND)][str(tmpInd)]
					htPerInd="/".join([str(val) for val in htVar])
					hlPerInd=",".join(\
						[str(HL[str(indexIND)][str(tmpInd)][var]) for var in trueRows])
					alleles=self.codifySequences(valuesToCodify)
					adPerInd=",".join(AD[str(indexIND)][alleles,indexVAR].astype(dtype=int).astype(dtype=np.str))

				ind="{0}:{1}:{2}:{3}".format(\
					htPerInd,\
					hlPerInd,\
					adPerInd,\
					DP[indexIND][indexVAR])
				allVariants[str(tmpInd)]+=[ind]

		return allVariants

	def codifySequences(self,seq):
		"""
		codify nucleotidic sequence in numbers
		A=0,C=1,G=2,T=3
		------------------------------------------------------------------------
		input:
			seq: nucleotidic sequence (containing only ACGT)
		output:
			codified nucleotidic sequence in numbers
		"""
		# self.appLogger.debug("codifySequences(self,ref)")
		codedRef=[]
		for item in seq:
			if "A"==item: codedRef+=[0]
			if "C"==item: codedRef+=[1]
			if "G"==item: codedRef+=[2]
			if "T"==item: codedRef+=[3]
		return np.array(codedRef)

	def writeVCFFile(self, indexREP,indexLOC,refAllelesFilePath,referenceDescription,REF,alt,variableSitesPositionIndices,HT,HL,AD,DP,flag):
		"""
		gets single individual data formated as a GT:GL:AD:DP VCF column
		------------------------------------------------------------------------
		intput:
			indexREP
			indexLOC
			refAllelesFilePath
			referenceDescription
			reference sequence
			alternative alleles
			variableSitesPositionIndices
			haplotype
			likelihood
			allelic depth
			read depth
			flag: to indicate whether is true o sampled what's being written
		output:
			string column with information in format: GT:GL:AD:DP
		"""
		self.appLogger.debug("writeVCFFile(self, indexREP,indexLOC,REF,alt,variableSites,HT,HL,AD,DP,flag)")
		# flag is either true or sampled
		nInds=len(HT.keys())
		if flag:
			self.appLogger.debug("Writing VCF file (no_error)")
		else:
			self.appLogger.debug("Writing VCF file (with_error)")
		header="{0}\r\n{1}={2}\r\n{3}\r\n{4}={5}".format(\
			"##fileformat=VCFv4.0",\
			"##fileDate",\
			datetime.datetime.now(),\
			"##source=ngsphy",
			"##reference",\
			refAllelesFilePath
		)
		formatLines="{0}\r\n{1}\r\n{2}\r\n{3}\r\n{3}".format(\
			"##FORMAT=<ID=GT,Number=1,Type=Integer,Description=\"Genotype\">",\
			"##FORMAT=<ID=PL,Number=1,Type=Integer,Description=\"Log10 scale genotype likelihood\">",\
			"##FORMAT=<ID=AD,Number=1,Type=Integer,Description=\"Allelic Depth\">",\
			"##FORMAT=<ID=DP,Number=1,Type=Integer,Description=\"Depth of coverage\">",\
			"##FILTER=<ID=NONE,Description=\"No filter applied\">"\
		)

		indnames=["Ind{0}".format(i) for i in range(0,nInds)]
		# filename, file_extension = os.path.splitext('/path/to/somefile.ext')
		headerCols=["#CHROM","POS","ID","REF","ALT","QUAL","FILTER","INFO","FORMAT"]+indnames
		# CHROM
		numGeneTreeDigits=len(str(self.numLociPerReplicate[(indexREP-1)]))
		chromName=referenceDescription
		# POS
		nVariants=len(variableSitesPositionIndices)
		POS=[(item+1) for item in variableSitesPositionIndices]
		# ID
		ID=["REP_{0:0{1}d}_GT_{2:0{3}d}_ID_{4}".format(\
			indexREP,\
			self.numReplicatesDigits,\
			indexLOC,
			numGeneTreeDigits,\
			ID) for ID in range(1, (nVariants+1))]
		# ALT
		ALT=[ ",".join(alt[str(pos)]) for pos in variableSitesPositionIndices ]
		# qual
		QUAL=[u'\u00B7']*nVariants
		# FILTER
		FILTER=["NONE"]*nVariants
		# INFO
		INFO=[u'\u00B7']*nVariants
		# format
		FORMAT=["GT:GL:AD:DP"]*nVariants
		# extra 9 columns: #CHROM,POS,ID,REF,ALT,QUAL,FILTER,INFO,FORMAT = 9
		nLoci=self.numLociPerReplicate[indexREP-1]
		numGeneTreeDigits=len(str(nLoci))
		allVariants=self.formatIndividualDataForVCF(\
			REF,alt,variableSitesPositionIndices,HT,HL,AD,DP)
		outfile=""
		# flag true=true  - Flag false= sampled
		if flag:
			outfile=os.path.join(
				self.readcountNoErrorFolderPath,\
				"REPLICATE_{0:0{1}d}".format(\
					indexREP,\
					self.numReplicatesDigits),\
				"{0}_{1:0{2}d}_{3:0{4}d}_NOERROR.vcf".format(\
					self.settings.simphyDataPrefix,\
					indexREP,self.numReplicatesDigits,\
					indexLOC,numGeneTreeDigits\
				)\
			)
		else:
			outfile=os.path.join(
				self.readcountErrorFolderPath,\
				"REPLICATE_{0:0{1}d}".format(\
					indexREP,\
					self.numReplicatesDigits),\
				"{0}_{1:0{2}d}_{3:0{4}d}.vcf".format(\
					self.settings.simphyDataPrefix,\
					indexREP,self.numReplicatesDigits,\
					indexLOC,numGeneTreeDigits\
				)\
			)


		# before writing i'm getting max width of the lines written per column
		colWidths=self.getColWidhts(\
			chromName,POS,ID,\
			REF,alt,QUAL,FILTER,\
			INFO,FORMAT,\
			allVariants,variableSitesPositionIndices)
		# (sizeChrom,sizePOS,sizeID,sizeREF,sizeALT,sizeQUAL,sizeFILTER,sizeINFO,sizeFORMAT,sizeInds)
		maxLenIndName=max([len(elem) for elem in indnames])
		maxLenIndName=max(colWidths[-1], maxLenIndName)
		headerWidths=[maxLenIndName]*len(indnames)
		headerWidths=colWidths+headerWidths
		# headerFields=["{0:{1}s}".format(\
		# 	headerCols[indexField],headerWidths[indexField])\
		# 	for indexField in range(0,len(headerCols))]
		headerFields=[headerCols[i] for i in range(0,len(headerCols))]
		# filevcf=codecs.open(outfile, 'a+', 'utf-8')
		filevcf=open(outfile, 'a')
		filevcf.write("{0}\r\n{1}\r\n{2}\r\n".format(\
			header,\
			formatLines,\
			"\t".join(headerFields)\
			))

		for index in range(0, nVariants):
			# extra 9 columns: #CHROM,POS,ID,REF,ALT,QUAL,FILTER,INFO,FORMAT = 9
			CHROM_FIELD="{0:{1}}".format(chromName,colWidths[0]).encode('utf-8').strip()
			# sys.stdout.write("{}\t".format(CHROM_FIELD))
			POS_FIELD="{0:{1}}".format(POS[index],colWidths[1]).encode('utf-8').strip()
			# sys.stdout.write("{}\t".format(POS_FIELD))
			ID_FIELD="{0:{1}}".format(ID[index],colWidths[2]).encode('utf-8').strip()
			# sys.stdout.write("{}\t".format(ID_FIELD))
			REF_FIELD="{0:{1}}".format(REF[variableSitesPositionIndices[index]],colWidths[3]).encode('utf-8').strip()
			# sys.stdout.write("{}\t".format(REF_FIELD))
			ALT_FIELD="{0:{1}}".format(",".join(alt[str(variableSitesPositionIndices[index])]).encode('utf-8'),colWidths[4])
			# sys.stdout.write("{}\t".format(ALT_FIELD))
			QUAL_FIELD='{0:{1}}'.format(QUAL[index].encode("utf-8"),colWidths[5])
			# sys.stdout.write("{}\t".format(QUAL_FIELD))
			FILTER_FIELD="{0:{1}}".format(FILTER[index].encode("utf-8"),colWidths[6])
			# sys.stdout.write("{}\t".format(FILTER_FIELD))
			INFO_FIELD="{0:{1}}".format(INFO[index].encode("utf-8"),colWidths[7])
			# sys.stdout.write("{}\t".format(INFO_FIELD))
			FORMAT_FIELD="{0:{1}}".format(FORMAT[index],colWidths[8]).encode('utf-8').strip()
			# sys.stdout.write("{}\t".format(FORMAT_FIELD))
			INDS_FIELD="\t".join(\
				['{0:{1}}'.format(indVar,maxLenIndName) for indVar in allVariants[str(variableSitesPositionIndices[index])]]\
			)
			# sys.stdout.write("\n{}\n".format(INDS_FIELD))
			line="{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}\t{9}\r\n".format(\
				CHROM_FIELD,POS_FIELD,ID_FIELD,REF_FIELD,ALT_FIELD,QUAL_FIELD,FILTER_FIELD,INFO_FIELD,FORMAT_FIELD,INDS_FIELD)
			filevcf.write(line)
		filevcf.close()

	def getColWidhts(self,chromName,POS,ID,REF,ALT,QUAL,FILTER,INFO,FORMAT,allVariants, variableSitesPositionIndices):
		"""
		get string widht of the columns to format the VCF output
		------------------------------------------------------------------------
		Parameters:
		- chromName: data column
		- POS: data column
		- ID: data column
		- REF: data column
		- ALT: data column
		- QUAL: data column
		- FILTER: data column
		- INFO: data column
		- FORMAT: data column
		- allVariants: data column
		- variableSitesPositionIndices: data column
		Returns:
		- list with the leghts of the columns that are going to be written in the VCF file
		"""
		self.appLogger.debug("getColWidhts(self,chromName,POS,ID,REF,ALT,QUAL,FILTER,INFO,FORMAT,allVariants, variableSites)")
		#CHROM,POS,ID,REF,ALT,QUAL,FILTER,INFO,FORMAT
		sizeChrom=len(chromName)
		sizePOS=max([len(str(item)) for item in POS]+[len("POS")])
		sizeID=max([len(item) for item in ID]+[len("ID")])
		sizeREF=max([len(item) for item in REF]+[len("REF")])
		tmpALT=[",".join(ALT[str(var)]) for var in variableSitesPositionIndices]
		sizeALT=max([len(item) for item in tmpALT]+[len("ALT")])
		sizeQUAL=max([len(item) for item in QUAL]+[len("QUAL")])
		sizeFILTER=max([len(item) for item in FILTER]+[len("FILTER")])
		sizeINFO=max([len(item) for item in INFO]+[len("INFO")])
		sizeFORMAT=max([len(item) for item in FORMAT]+[len("FORMAT")])
		tmpInds=[]
		for item in variableSitesPositionIndices:
			tmpInds+=[max([ len(str(elem)) for elem in allVariants[str(item)]])]
		sizeInds=max(tmpInds)
		return [sizeChrom,sizePOS,sizeID,sizeREF,sizeALT,sizeQUAL,sizeFILTER,sizeINFO,sizeFORMAT,sizeInds]

	def writeReference(self,indexREP,indexLOC,referenceSpeciesID,referenceLocusID,referenceTipID,referenceSeqFull):
		"""
		write reference sequence file separately
		------------------------------------------------------------------------
		Parameters:
		- indexREP: index of the species tree to which it belongs
		- indexLOC: index of the gene tree to which it belongs
		- referenceSpeciesID: species ID to which the reference belongs within a MSA file
		- referenceTipID:  tip ID to which the reference belongs within a species in a MSA file
		- referenceSeqFull: nucleotidic sequence
		Returns:
		- filepath where the reference will be written.
		"""
		self.appLogger.debug(" writeReference(self,indexREP,indexLOC,referenceSpeciesID,referenceTipID,referenceSeqFull):")
		refAllelesFilePath=os.path.join(\
			self.settings.refAllelesFolderPath,\
			"REPLICATE_{0:0{1}d}".format(indexREP,self.numReplicatesDigits),\
			"{0}_REF_{1}_{2}.fasta".format(\
				self.settings.projectName,\
				indexREP,
				indexLOC\
			)\
		)
		description=">REFERENCE:{0}:ST.{1:{2}}:GT.{3:{4}}:{5}_{6}_{7}".format(\
			self.settings.simphyDataPrefix,\
			indexREP,\
			self.numReplicatesDigits,\
			self.numLociPerReplicate[(indexREP-1)],\
			len(str(self.numLociPerReplicate[(indexREP-1)])),\
			referenceSpeciesID,\
			referenceLocusID,\
			referenceTipID\
		)
		f=open(refAllelesFilePath,"w")
		f.write("{0}\n{1}\n".format(\
			description,\
			"".join(referenceSeqFull)
		))
		return refAllelesFilePath



	def launchCommand(self, referenceForCurrST, indexREP,indexLOC, individuals,coverageMatrix):
		"""
		Launches the execution ogf the RC computation.
		------------------------------------------------------------------------
		Parameters:
		- referenceForCurrST: reference for current species tree replicate, the
		 one that  will be processed
		- indexREP: proper identifier of the species tree replicete
		- indexLOC: proper identifier of the gene tree
		- individuals: number of individuals
		- coverageMatrix: coverage matrix
		"""
		try:
			self.appLogger.debug("launchCommand(...)")
			tStartTime=datetime.datetime.now()
			numIndividuals=len(individuals.keys())
			nLoci=self.numLociPerReplicate[indexREP-1]
			self.appLogger.debug("Iterating over nLoci: {0}/{1}".format(indexLOC,nLoci))
			numGeneTreeDigits=len(str(nLoci))
			filepathLoc=os.path.join(\
				self.settings.basepath,\
				"{0:0{1}d}".format(indexREP,self.numReplicatesDigits),\
				"{0}_{1:0{2}d}_TRUE.fasta".format(\
					self.settings.simphyDataPrefix,\
					indexLOC,\
					numGeneTreeDigits\
				)\
			)

			# dictionary. indices=variable sites, content=variable nucleotide set
			self.appLogger.debug(\
				"Extracting variable positions from Replicate - {0:0{1}d}/ Locus - {2:0{3}d}".format(\
				indexREP, self.numReplicatesDigits, indexLOC, self.numLociPerReplicateDigits[indexREP-1]
			))
			variableSites=self.extractTrueVariantsPositions(filepathLoc)
			nVarSites=len(variableSites.keys())
			self.appLogger.debug(\
				"Species tree: {0}\t Locus: {1}\t - Found [{2}] variable sites.".format(\
					indexREP,\
					indexLOC,\
					nVarSites))
			# Parse MSA files - dictionary[speciesID][tipID]
			#				   content={description,sequence}
			msa=parseMSAFile(filepathLoc)
			# indices to get the sequence of the REFERENCE!
			tag="{0}_{1}".format(referenceForCurrST[1],referenceForCurrST[2])
			if tag in msa.keys():
				referenceSpeciesID=str(referenceForCurrST[1])
				referenceLocusID=str(referenceForCurrST[2])
				referenceTipID=str(referenceForCurrST[3])
				referenceDescription="_".join([str(i) for i in referenceForCurrST[1:4]])
				referenceSeqFull=msa[tag][referenceTipID]['sequence']
				refAllelesFilePath=self.writeReference(\
					indexREP,indexLOC,\
					referenceSpeciesID,referenceLocusID,referenceTipID,\
					referenceSeqFull)
				# Coverage per SNP is the same for both true/sampled dataset
				self.appLogger.info("{0} {1}\t Locus: {2}\t - {3}".format(\
					"Replicate:",\
					indexREP,\
					indexLOC,\
					"Getting coverage per individual per variant"\
				))
				DP=[self.getDepthCoveragePerIndividual(\
						nVarSites,\
						coverageMatrix[index][(indexLOC-1)]) \
						for index in range(0,numIndividuals)\
						]
				if self.settings.ploidy==1:
					self.computeHaploid(\
						indexREP,indexLOC,\
						msa,individuals,\
						refAllelesFilePath,\
						referenceDescription,\
						referenceSeqFull,\
						variableSites,DP)
				if self.settings.ploidy==2:
					self.computeDiploid(\
						indexREP,indexLOC,\
						msa,individuals,\
						refAllelesFilePath,\
						referenceDescription,\
						referenceSeqFull,\
						variableSites,DP)
				tEndTime=datetime.datetime.now()
				ETA=(tEndTime-tStartTime).total_seconds()
				if(self.settings.runningTimes):
					outfile=os.path.join(
						self.readcountNoErrorFolderPath,\
						"REPLICATE_{0:0{1}d}".format(\
							indexREP,\
							self.numReplicatesDigits),\
						"{0}_{1:0{2}d}_{3:0{4}d}*".format(
							self.settings.simphyDataPrefix,\
							indexREP,\
							self.numReplicatesDigits,\
							indexLOC,\
							numGeneTreeDigits\
						)\
					)
					line=[indexREP,indexLOC,"-",filepathLoc,ETA,"-",outfile]
	 				self.runningInfo.addLine(line)
		except KeyboardInterrupt:
			message="{0}{1}Thread has been interrupted!{2}\n".format("\033[91m","\033[1m","\033[0m")
			raise KeyboardInterrupt(message)

	def run(self):
		"""
		Process flow for the generation of read counts
		------------------------------------------------------------------------
		Returns:
		- Boolean indating whether process has finished correctly or not.
		"""
		self.appLogger.debug("run(self)")
		status=True;	message="Read counts finished ok."
		self.appLogger.debug( "Run - read count")
		try:
			# generating folder structure for this part
			self.generateFolderStructureDetail()
			# Get list of reference sequences
			referenceList=self.parseReferenceAllelesList(self.settings.readCountsReferenceAllelesFile)
			nSTS=len(self.filteredReplicates)
			# iterate over the "iterable" species trees / filtered STs
			self.appLogger.info("Running...")
			nProcesses=sum([self.numLociPerReplicate[item-1] for item in self.filteredReplicates])
			currProcessesRunning=1
			pool=multiprocessing.Pool(self.settings.numThreads)

			filepathIndividualsRelations=[\
				os.path.join(\
					self.settings.tablesFolderPath,\
					"{0}.{1:0{2}d}.individuals.csv".format(\
					self.settings.projectName,\
					indexREP,\
					self.numReplicatesDigits\
					)\
				)
			for indexREP in self.filteredReplicates]

			individualsList=[self.parseIndividualRelationFile(singlefile) for singlefile in filepathIndividualsRelations]
			numIndividualsList=[len(item.keys()) for item in individualsList]
			coverageMatrices=[self.retrieveCoverageMatrix(indexREP) for indexREP in self.filteredReplicates]
			nLociList=[self.numLociPerReplicate[item-1] for item in self.filteredReplicates]
			ARGS=[(referenceList[item],\
				self.filteredReplicates[item],\
				nLociList[item],\
				individualsList[item],\
				coverageMatrices[item])\
			for item in range(0,nSTS)]

			for indexFilterST in range(0,nSTS):
				indexREP=self.filteredReplicates[indexFilterST] # Get Proper ID for ST
				nLoci=nLociList[indexFilterST]
				self.appLogger.debug("Number of individuals: {0}".format(numIndividualsList[indexFilterST]))
				self.appLogger.debug("Number of loci: {0}".format(nLoci))

				indexLOC=1
				threadsToBeRan=nLoci
				self.appLogger.info("Simulating read counts for Replicate - {0:0{1}d}".format(\
					indexREP, self.numReplicatesDigits))
				while indexLOC < (nLoci+1):
					# for indexLOC in range(1,(nLoci+1)):
					jobs=[]
					for item in range(0,min(self.settings.numThreads, threadsToBeRan)):
						progress=((currProcessesRunning*100)*1.0)/(nProcesses+1)
						currProcessesRunning+=1
						sys.stdout.write("Progress {0:02.1f} %\r".format(progress))
						sys.stdout.flush()
						time.sleep(0.2)
						t = multiprocessing.Process(\
							target=self.launchCommand,\
							args=(\
								referenceList[indexREP-1],\
								indexREP,\
								indexLOC,\
								individualsList[indexFilterST],\
								coverageMatrices[indexFilterST])\
						)
						t.daemon=True
						jobs.append(t)
						t.start()
						indexLOC+=1
						threadsToBeRan=(nLoci+1)-indexLOC

					for j in jobs:
						j.join()

		except ValueError as ve:
			# If there's a wrong ploidy inserted.
			status=False
			message="\n\t{0}\n\t{1}\n\t{2}".format(\
				"Ploidy value invalid",\
				ve,\
				"Please verify. Exiting.")
		except TypeError as te:
			# One of the files is not fasta.
			status=False
			message=message="\n\t{0}\n\t{1}\n\t{2}".format(\
				"Problem with the file type.",\
				te,\
				"Please verify. Exiting.")
		except Exception as ex:
			exc_type, exc_obj, exc_tb = sys.exc_info()
			fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
			message="\n\tUnexpected: {0} | {1} - File: {2} - Line:{3}".format(\
				ex,exc_type, fname, exc_tb.tb_lineno)
			status=False
		return status,message



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import argparse,datetime,dendropy,glob, logging,msatools,os,re,random,sys
import numpy as np
from coverage import NGSPhyDistributionParser as ngsphydistro
if (sys.version_info[0:2]<(3,0)):
	import ConfigParser as cp
elif (sys.version_info>=(3,0)):
	import configparser as cp

class Settings:
	"""
	This class parses and verifies that the settings file is correct.
	----------------------------------------------------------------------------
	Attributes:

	General
	-------
	- inputmode: Represents where the input data comes from. Has 3 different values.
	(1) when using the output of a SimPhy runART. (2) when using a single gene tree
	with a random root/reference sequence and (3) when using a single gene tree with a
	specific root/reference sequence.
	- ploidy: This is to identify whether the user wants to generate haploid or
	diploid individuals from the sequences given/generated. Not all ploidies are
	accepted for all oriiginModes. Please verify the external documentation for a
	detail explanation.
	- projectName: NGSPHY or the name of the SimPhy folder.
	- path: indicates where the output folder is/will be created.
	- outputFolderName: Name of the folder that will be used to store the generated
	data.
	- outputFolderPath: absolute path of the output folder.
	- simphyDataPrefix: Prefix of the sequences. If generated within NGSphy, default is
	"ngsphydata", otherwise, this informations is parsed from the settings
	file and will have to correspond, to the prefix given to SimPhy.
	- simphyFolderPath: Related to InputMode=1. Corresponds to the absolute path
	of the SimPhy project.
	- simphyFilter:  Related to InputMode=1. Using species tree distributions.
	Allows the user to filter out from the program execution, those species tree
	replicates that do not match the given ploify to the number of individuals
	per species to be able to generate individuals according to the given ploidy.


	- indelibleControlFile: Related to InputMode in [2,3]. Path
	to the modified INDELible control file.
	- geneTreeFile: Related to InputMode=2. Path to the gene tree file in
	Newick format. Must contain a single tree. Name of the file without extension
	must match the name of the tree within the control file.
	- evolve: Related to InputMode=2. To handle evolve parameters of the control
	file.
	- partition: Related to InputMode=2. To handle partition parameters of the
	control file.


	- ancestralSequenceFilePath: Related to InputMode=3. Path to the reference
	sequene file where the root/reference sequence is.
	- anchorTipLabel: Related to InputMode=3. Label of the tip that will be
	used as reference/root.

	- readCountsError: Related to NGSMode=1. sequencing read_counts_error that will be used for the read counts.
	- readCountsReferenceAllelesFile: Related to NGSMode=1. Path to the file that contains all the
	information (ReplicateIdentifier, Species, Locus, Gene) needed to
	select a sequence and use it as a reference to generate the read counts.

	- ngsmode: Variable that identifies which ngsmode will be used.

	- ontarget: Parameter related to coverage. Only possible with inputmode=1.
	When emulating Targeted-sequencing, represents the fraction of the loci that
	will be considered as on-target.
	- offtarget: Parameter related to coverage.  Only possible with inputmode=1.
	When emulating Targeted-sequencing, represents the fraction of the loci that
	will be considered as on-target.
	- notcaptured: Parameter related to coverage.  Only possible with inputmode=1.
	When emulating Targeted-sequencing, represents the fraction of the on-target
	loci that will not be sequece, coverage == 0.
	- experiment: Parameter related to coverage. Value for the expected coverage.
	Parameterization can be based on a distribution or a fixed value.
	- individual: Parameter related to coverage. Expected coverage multiplier. Hyper
	parameter. Value used here can be based on a distribution or a fixed value.
	Value sampled will be used as the alpha shape of a Gamma distribution with
	mean = 1.
	- locus: Parameter related to coverage. Expected coverage multiplier. Hyper
	parameter. Value used here can be based on a distribution or a fixed value.
	Value sampled will be used as the alpha shape of a Gamma distribution with
	mean = 1.
	- taxon: Parameter related to coverage. This parameters is to
	represent the possible coverage decay due to the distance of the species
	to the reference sequence within the given tree.
	Is a list of pairs, where the first element is the species identifier and
	the second, is the fraction of coverage that the species will keep.

	- numReplicates: number of species tree replicates.
	- numLociPerReplicate: digits used to represente the numReplicates
	- indels: Indicates whether the sequences (given or generated) contain
	indels. Important for ngsmode=1 (ReadCounts), because this mode does not
	handle sequence with indels.
	"""
	# General
	inputmode=1
	ploidy=1
	projectName="NGSphy"
	path=""
	outputFolderName="NGSphy_output"
	outputFolderPath=""
	basepath=""


	# simphy data origin | origin 1
	simphyDataPrefix="ngsphydata"
	simphyFolderPath=""
	simphyFilter=False

	programCommand=""
	# indelible data origin | origin 2
	indelibleControlFile=""
	geneTreeFile=""
	evolve=""
	partition=""
	# indelible data origin + referebce | origin 3
	ancestralSequenceFilePath=""
	anchorTipLabel=""

	#readcount
	readCountsError=0
	readCountsReferenceAllelesFile=""

	ngsmode=0
	coveragemodeART=False # FALSE is FCOV/F - TRUE C/RCOUNTS

	# coverage
	ontarget=1
	offtarget={"loci":0, "coverage":1}
	notcaptured=0
	experiment=None
	individual=None
	locus=None
	taxon=dict()

	numReplicates=0
	numLociPerReplicate=[]
	indels=False

	alignmentsFolderPath=""
	coverageFolderPath=""
	individualsFolderPath=""
	readsFolderPath=""
	refAllelesFolderPath=""
	scriptsFolderPath=""
	tablesFolderPath=""

	runART=False
	threads=1
	executionMode=1

	__NUCLEOTIDES=["A","C","G","T"]
	def __init__(self,filename):
		# If I've got this far, then filename is a correct file
		self.settingsFile=os.path.abspath(filename)
		self.appLogger=logging.getLogger("ngsphy")
		self.appLogger.debug("(class Settings) __init__()")
		# default settings can be established.
		self.parser=cp.SafeConfigParser()
		self.parser.read(self.settingsFile)

	def checkArgs(self):
		"""
		Checks the existence and validity of the settings introduced in the
		settings file per block and option.s
		------------------------------------------------------------------------
		Returns:
		- boolean, message: the status of the process and the message related to
		such status
		"""
		self.appLogger.debug("Check arguments")
		allGood=True
		parserMessageCorrect="All parameters are correct."
		parserMessageWrong="Settings - Problem found! "
		# Checking general block
		statusGeneral,messageGeneral= self.checkBlockGeneral(\
			parserMessageCorrect,parserMessageWrong)
		# Exit here
		if not statusGeneral: return statusGeneral, messageGeneral

		# Checking data block
		statusData,messageData=self.checkBlockData(\
			parserMessageCorrect,parserMessageWrong)
		# Exit here
		if not statusData: return statusData, messageData
		# ----------------------------------------------------------------------
		# NGS sections
		if self.parser.has_section("ngs-reads-art"):
			self.ngsmode=1
			statusNGSArt,messageNGSArt=self.checkBlockNGSReadsArt(\
				parserMessageCorrect,parserMessageWrong)
			if (statusNGSArt):
				## Next check
					statusCoverage,messageCoverage=self.checkBlockCoverage(parserMessageCorrect,parserMessageWrong)
					# Exit here
					if not statusCoverage: return statusCoverage, messageCoverage
			else:
				# Exit here
				return statusNGSArt,messageNGSArt
			# if all parameters for NGS ART are correct I don't need the other NGS block
			# removing NGS read count
			if self.parser.has_section("ngs-read-counts"):
				self.parser.remove_section("ngs-read-counts")
				self.appLogger.warning("[ngs-read-counts] block is incompatible with [ngs-reads-art]. Omiting this block.")
		elif self.parser.has_section("ngs-read-counts"):
			self.ngsmode=2
			# readcount
			statusRC,messageRC=self.checkBlockNGSReadCounts()
			if statusRC:
				statusCoverage,messageCoverage=self.checkBlockCoverage(parserMessageCorrect,parserMessageWrong)
				# Exit here
				if not statusCoverage: return statusCoverage, messageCoverage
			else:
				# Exit here
				return statusRC,messageRC
		else: # Meaning have no ngs-reads-art nor ngs-read-countss
			if (self.parser.has_section("coverage")):self.parser.remove_section("coverage")
			self.appLogger.info("[coverage] block is not needed if [ngs-reads-art] or [ngs-read-counts] are not available. Omiting this block.")

		# Checking execution block and options
		self.checkBlockExecution(parserMessageCorrect,parserMessageWrong)
		# Exit here
		self.appLogger.info(self.formatSettingsMessage())


		return allGood,parserMessageCorrect

	def checkBlockGeneral(self,parserMessageCorrect,parserMessageWrong):
		"""
		Checks the General block of the settings, regarding general paths,
		project name and ploidy.
		------------------------------------------------------------------------
		Returns:
		- boolean, message: the status of the process and the message related to
		such status
		"""
		self.appLogger.debug("Block General")
		# Check GENERAL SECTION
		if not self.parser.has_section("general"):
			parserMessageWrong="\n\t{0}\n\t{1}".format(\
				"[general] block missing and required.",\
				"Please verify. Exiting."\
			)
			return False, parserMessageWrong

		# Checking seeding
		if (self.parser.has_option("general","seed")):
			self.seed=self.parser.getint("general","seed")
			if (self.seed > ((2**32)-1)) or (self.seed < 0 ):
				self.seed=np.random.random_integers(0,(2**32) - 1)
		else:
			self.seed=np.random.random_integers(0,(2**32) - 1)
		random.seed(self.seed)
		np.random.seed(seed=self.seed)

		# CHECKING GENERAL PARAMETERS
		# checking ploidy for the output data
		if (not self.parser.has_option("general","ploidy")):
			self.ploidy=1
		else:
			p=self.parser.getint("general","ploidy")
			if p in [1,2]:
				self.ploidy=p
			else:
				parserMessageWrong="\n\t{0}\n\t{1}\n\t{2}".format(\
					"Ploidy value is out of range.",\
					"Value must be in [1,2].",\
					"Please verify. Exiting."\
				)
				return False, parserMessageWrong
		if (self.parser.has_option("general","path")):
			self.path=os.path.abspath(self.parser.get("general","path"))
		else:
			parserMessageWrong="\n\t{0}\n\t{1}".format(\
				"Path option is needed.",\
				"Please verify. Exiting."
			)
			return False, parserMessageWrong

		# Checking output folder information
		if(self.parser.has_option("general","output_folder_name")):
			self.outputFolderName=self.parser.get("general","output_folder_name")

		if os.path.exists(os.path.join(self.path, self.outputFolderName)):
			counter=len(glob.glob("{0}*".format(os.path.join(self.path, self.outputFolderName))))
			self.outputFolderName+="_{0}".format(counter+1)

		self.outputFolderPath=os.path.join(self.path,self.outputFolderName)
		self.alignmentsFolderPath=os.path.join(\
			self.outputFolderPath,"alignments")
		self.coverageFolderPath=os.path.join(\
			self.outputFolderPath,"coverage")
		self.individualsFolderPath=os.path.join(\
			self.outputFolderPath,"individuals")
		self.readsFolderPath=os.path.join(\
			self.outputFolderPath,"reads")
		self.refAllelesFolderPath=os.path.join(\
			self.outputFolderPath,"ref_alleles")
		self.scriptsFolderPath=os.path.join(\
			self.outputFolderPath,"scripts")
		self.tablesFolderPath=os.path.join(\
			self.outputFolderPath,"ind_labels")

		return True,parserMessageCorrect

	def checkBlockData(self,parserMessageCorrect,parserMessageWrong):
		"""
		Checks Data block of the settings file, regarding origin and parameters
		related to each origin mode.
		------------------------------------------------------------------------
		Returns:
		- boolean, message: the status of the process and the message related to
		such status
		"""
		self.appLogger.debug("Block Data")
		if (self.parser.has_section("data")):
			if (self.parser.has_option("data","inputmode")):
				try:
					self.inputmode=self.parser.getint("data","inputmode")
				except:
					parserMessageWrong="\n\t\n\t{0}\n\t{1}\n\t{2}".format(\
						"[data] block: Value introduced in the inputmode option is not valid.",\
						"Value should be an integer.",\
						"Please verify. Exiting."\
					)
					return False, parserMessageWrong
			else:
				parserMessageWrong="\n\t\n\t{0}\n\t{1}\n\t{2}".format(\
					"[data] block: inputmode option value is missing. This option is required.",\
					"Please verify. Exiting."\
				)
				return False, parserMessageWrong

			####################################################################
			if not self.inputmode in [1,2,3,4]:
				parserMessageWrong="\n\t\n\t{0}\n\t{1}".format(\
					"[data] block: inputmode option value is invalid",\
					"Value is out of range. Possible range [1,3]",\
					"Please verify. Exiting."\
				)
				return False, parserMessageWrong
			####################################################################

			# SINGLE GENE TREE MODES GENERAL
			if self.inputmode < 4: # 1,2,3
				self.basepath=self.alignmentsFolderPath
					# parameter is set up, now check if folder exist
				if (self.parser.has_option("data","indelible_control_file")):
					self.indelibleControlFile=os.path.abspath(\
						self.parser.get("data","indelible_control_file")
					)
				else:
					parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(\
						"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
						"INDELible  control file is required",\
						"Please verify. Exiting."\
					)
					return False, parserMessageWrong
				# checkin indelible_control_file file
				if (os.path.exists(self.indelibleControlFile) and os.path.isfile(self.indelibleControlFile)):
					self.appLogger.debug("INDELible control file exists")
				else:
					parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(\
						"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
						"INDELible control file does not exist.",\
						"Please verify. Exiting."\
					)
					return False, parserMessageWrong

				# checkin tree file
				if (self.parser.has_option("data","gene_tree_file")):
					self.appLogger.debug("Gene tree file exists")
					self.geneTreeFile=os.path.abspath(self.parser.get("data","gene_tree_file").strip())
					filesOk=(os.path.exists(self.geneTreeFile) and os.path.isfile(self.geneTreeFile))
					if not filesOk:
						parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(\
							"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
							"Gene tree file does not exist.",\
							"Please verify. Exiting."\
						)
						return False, parserMessageWrong
				else:
					parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(\
						"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
						"Gene tree file does not exist.",\
						"Please verify. Exiting."\
					)
					return False, parserMessageWrong


				self.appLogger.debug("Checking INDELible control file")
				# checking control file format
				statusOk, message=self.checkIndelibleControlFile(parserMessageCorrect,parserMessageWrong)
				if not statusOk: return statusOk,message
				# checking tip label format
				self.appLogger.debug("Checking tree label formatting")
				statusOk, message=self.checkLabelFormatInTree()
				if not statusOk: return statusOk,message
				statusOk, message=self.checkBranchLengthsInTree()
				if not statusOk: return statusOk,message

				self.parser.set("general","numreplicates",str(1))
				self.numReplicates=1
				# removing options that do not match with the origin mode selected
				if self.parser.has_option("data","simphy_data_prefix"): self.parser.remove("data","simphy_data_prefix")
				if self.parser.has_option("data","simphy_folder_path"): self.parser.remove("data","simphy_folder_path")
				if self.parser.has_option("data","simphy_filter"):  self.parser.remove("data","simphy_filter")

			####################################################################
			if self.inputmode ==1:
				# checking indelible program
				stream1 = os.popen('which indelible').read()[0:-1]
				stream2 = os.popen('which indelible-ngsphy').read()[0:-1]
				self.appLogger.info("Checking dependencies...")
				if stream1:
					self.appLogger.info("indelible - Found running in: {}".format(stream1))
					self.programCommand="indelible"
				elif stream2:
					self.appLogger.info("indelible-ngsphy - Found running in: {}".format(stream2))
					self.programCommand="indelible-ngsphy"
				else:
					parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(\
						"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
						"indelible nor indelible-ngsphy found. Programs either not installed or not in the current path.",\
						"Please verify the installation. Exiting."\
					)
					return False, parserMessageWrong
				if self.parser.has_option("data","ancestral_sequence_file"):  self.parser.remove_option("data","ancestral_sequence_file")
				if self.parser.has_option("data","anchor_sequence_file"):  self.parser.remove_option("data","anchor_sequence_file")
				if self.parser.has_option("data","anchor_tip_label"):  self.parser.remove_option("data","anchor_tip_label")
			####################################################################
			if self.inputmode in [2,3]:
				stream = os.popen('which indelible-ngsphy').read()[0:-1]
				self.appLogger.info("Checking dependencies...")
				if stream:
					self.appLogger.info("indelible-ngsphy - Found running in: {}".format(stream))
					self.programCommand="indelible-ngsphy"
				else:
					parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(\
						"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
						"indelible-ngsphy not found. Program either not installed or not in the current path.",\
						"Please verify the installation. Exiting."\
					)
					return False, parserMessageWrong
			####################################################################
			if self.inputmode==2:
				if (self.parser.has_option("data","ancestral_sequence_file")):
					self.ancestralSequenceFilePath=os.path.abspath(self.parser.get("data","ancestral_sequence_file").strip())
					filesOk=(os.path.exists(self.ancestralSequenceFilePath) and os.path.isfile(self.ancestralSequenceFilePath))
					if not filesOk:
						parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(
						"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
						"Ancestral sequence file does not exist or the given path does not belong to a file.",\
						"Please verify. Exiting."\
						)
						return False, parserMessageWrong
					statusSeq,messageSeq=self.correctContentReferenceSequence()
					if not statusSeq: return statusSeq, messageSeq
				else:
					parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(\
					"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
					"Ancestral sequence file is required",\
					"Please verify. Exiting."\
					)
					return False, parserMessageWrong
				if self.parser.has_option("data","anchor_sequence_file"):  self.parser.remove_option("data","anchor_sequence_file")
			####################################################################
			if self.inputmode==3:
				if (self.parser.has_option("data","anchor_sequence_file")):
					self.ancestralSequenceFilePath=os.path.abspath(self.parser.get("data","anchor_sequence_file").strip())
					filesOk=(os.path.exists(self.ancestralSequenceFilePath) and os.path.isfile(self.ancestralSequenceFilePath))
					if not filesOk:
						parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(
						"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
						"Anchor sequence file does not exist or the given path does not belong to a file.",\
						"Please verify. Exiting."\
						)
						return False, parserMessageWrong
					statusSeq,messageSeq=self.correctContentReferenceSequence()
					if not statusSeq: return statusSeq, messageSeq
				else:
					parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(\
					"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
					"Anchor sequence file is required",\
					"Please verify. Exiting."\
					)
					return False, parserMessageWrong
				if (self.parser.has_option("data","anchor_tip_label")):
					self.anchorTipLabel=self.parser.get("data","anchor_tip_label").strip()
					statusOk, message=self.checkAnchorTipLabelInTree()
					if not statusOk: return statusOk,message
				else:
					parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(\
						"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
						"Anchor tip label is required",\
						"Please verify. Exiting."\
					)
					return False, parserMessageWrong
				if self.parser.has_option("data","ancestral_sequence_file"):  self.parser.remove_option("data","ancestral_sequence_file")
			####################################################################
			if self.inputmode==4:
				if (self.parser.has_option("data","simphy_folder_path")):
					self.simphyFolderPath=os.path.abspath(self.parser.get("data","simphy_folder_path"))
					self.basepath=self.simphyFolderPath
				else:
					parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(\
						"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
						"SimPhy folder does not exist, or the given path does not belong to a directory.",\
						"Please verify. Exiting."\
					)
					return False, parserMessageWrong
				if (os.path.exists(self.simphyFolderPath) and os.path.isdir(self.simphyFolderPath)):
					self.projectName=os.path.basename(self.simphyFolderPath)
				else:
					parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(\
						"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
						"SimPhy folder does not exist, or the given path does not belong to a directory.",\
						"Please verify. Exiting."\
					)
					return False, parserMessageWrong
				# data prefix for the simphy sequences
				if (self.parser.has_option("data","simphy_data_prefix")):
					self.simphyDataPrefix=self.parser.get("data","simphy_data_prefix")
				else:
					parserMessageWrong="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(\
						"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
						"<simphy_data_prefix> field is missing. This prefix correponds to the name of the file sequences that are going to be processed.",\
						"Please verify. Exiting."\
					)
					return False, parserMessageWrong
				# checking flag for filtering species tree replicates
				if self.parser.has_option("data","simphy_filter"):
					self.simphyFilter=True
				else:
					self.simphyFilter=False
				# removing options that do not match with the origin mode selected
				if (self.parser.has_option("data","indelible_control_file")): self.parser.remove_option("data","indelible_control_file")
				if (self.parser.has_option("data","gene_tree_file")): self.parser.remove_option("data","gene_tree_file")
				if (self.parser.has_option("data","anchor_tip_label")): self.parser.remove_option("data","anchor_tip_label")
				if (self.parser.has_option("data","ancestral_sequence_file")): self.parser.remove_option("data","ancestral_sequence_file")
				if (self.parser.has_option("data","anchor_sequence_file")): self.parser.remove_option("data","anchor_sequence_file")
				# --------------------------------------------------------------
				# check if simphy is valid projectName
				####################################################################
				status,message=self.checkSimPhyProjectValid()
				if not status: return status, message
		else:
			parserMessageWrong="\n\t\n\t{0}\n\t{1}\n\t{2}".format(\
				"[data] block is missing. This block is required.",\
				"Please verify. Exiting."\
			)
			return False, parserMessageWrong
		####################################################################

		return True, parserMessageCorrect


	def checkBlockNGSReadsArt(self,parserMessageCorrect,parserMessageWrong):
		"""
		Checks NGS-reads-ART block of the settings file, regarding parameters
		of the NGS simulation with ART
		------------------------------------------------------------------------
		Returns:
		- boolean, message: the status of the process and the message related to
		such status
		"""
		########################################################################
		# BLOCK: NGS-READS-ART
		########################################################################
		# Checking art parameters.
		self.appLogger.info("NGS-reads-ART option selected.")
		# checking program dependencies
		stream = os.popen('which art_illumina').read()[0:-1]
		self.appLogger.info("Checking dependencies...")
		if stream:
			self.appLogger.info("art_illumina - Found running in: {}".format(stream))
			# IO parameters
			if self.parser.has_option("ngs-reads-art","o"):self.parser.remove_option("ngs-reads-art","o")
			if self.parser.has_option("ngs-reads-art","out"):self.parser.remove_option("ngs-reads-art","out")
			if self.parser.has_option("ngs-reads-art","i"):self.parser.remove_option("ngs-reads-art","i")
			if self.parser.has_option("ngs-reads-art","in"):self.parser.remove_option("ngs-reads-art","in")
			self.appLogger.warning("Removing I/O options. Be aware: I/O naming is auto-generated.")
			if (not self.parser.has_option("ngs-reads-art","fcov")   and
					not self.parser.has_option("ngs-reads-art","f")  and
					not self.parser.has_option("ngs-reads-art","rcount") and
					not self.parser.has_option("ngs-reads-art","c") ):
				parserMessageWrong="\n\t{0}\n\t{1}\n\t{2}".format(\
					"There has been a problem with the coverage process parameterization.",\
					"One of the f/fcov or c/rcount parameteres needed.",
					"Please verify. Exiting."
				)
				return False, parserMessageWrong
			# # Coverage parameters
			if (self.parser.has_option("ngs-reads-art","fcov")):
				self.parser.remove_option("ngs-reads-art","fcov")
				self.coveragemodeART=False
			if (self.parser.has_option("ngs-reads-art","f")):
				self.parser.remove_option("ngs-reads-art","f")
				self.coveragemodeART=False
			if (self.parser.has_option("ngs-reads-art","rcount")):
				self.parser.remove_option("ngs-reads-art","rcount")
				self.coveragemodeART=True
			if (self.parser.has_option("ngs-reads-art","c")):
				self.parser.remove_option("ngs-reads-art","c")
				self.coveragemodeART=True
			# self.appLogger.warning("Removing ART coverage options. Coverage is calculated with the [coverage] block parameters.")
		else:
			parserMessageWrong="\n\t{0}\n\t{1}".format(\
				"art_illumina not found. Program either not installed or not in your current path.",\
				"Please verify the installation. Exiting."
			)
			return False, parserMessageWrong
		return True, parserMessageCorrect

	def checkBlockNGSReadCounts(self):
		"""
		Checks read counts  block of the settings file, regarding parameters
		of the read counts process.
		------------------------------------------------------------------------
		Returns:
		- boolean, message: the status of the process and the message related to
		such status
		"""
		########################################################################
		# BLOCK: READ COUNT
		########################################################################
		message=""; status=True
		if (self.parser.has_section("ngs-read-counts")):
			if not self.parser.has_option("ngs-read-counts", "read_counts_error"):
				self.appLogger.warning("[ngs-read-counts] block. Read counts error rate for this runART is being considered as 0.")
				self.parser.set("ngs-read-counts", "read_counts_error","0")
				self.readCountsError=0
			else:
				try:
					self.readCountsError=float(self.parser.get("ngs-read-counts", "read_counts_error"))
				except:
					message="\n\t{0}\n\t{1}".format(\
						"[ngs-read-counts] block."
					)
					return False, message

			if not self.parser.has_option("ngs-read-counts","reference_alleles_file"):
				self.appLogger.warning("[ngs-read-counts] block. Using default references.")
				self.parser.set("ngs-read-counts", "reference_alleles_file","None")
				self.readCountsReferenceAllelesFile=None
			else:
				self.readCountsReferenceAllelesFile=os.path.abspath(self.parser.get("ngs-read-counts","reference_alleles_file"))
				fileOk=os.path.exists(self.readCountsReferenceAllelesFile) and os.path.isfile(self.readCountsReferenceAllelesFile)
				if not fileOk:
					status=False
					message="\n\t{0}\n\t{1}\n\t{2}".format(\
						"[ngs-read-counts] block.",\
						"Reference file does not exist or path is incorrect.",\
						"Please verify. Exiting.")
		else:
			# No ngs-read-counts block
			status=False
			message="\n\t{0}\n\t{1}".format(\
				"[ngs-read-counts] block is missing.",\
				"Please verify. Exiting.")

		return status,message

	########################################################################
	# BLOCK: Coverage
	########################################################################
	def checkBlockCoverage(self,parserMessageCorrect,parserMessageWrong):
		"""
		Checks coverage block of the settings file, regarding parameters that
		model the coverage variation within the execution
		------------------------------------------------------------------------
		Returns:
		- boolean, message: the status of the process and the message related to
		such status
		"""
		self.appLogger.debug("Checking coverage")
		message=parserMessageCorrect
		expCov=None;indCov=None;locCov=None;
		if(self.parser.has_section("coverage")):
			# ------------------------------------------------------------------
			#  Targeted-sequencing related
			if self.inputmode==4:
				self.appLogger.debug("Inputmode 4")
				if (self.parser.has_option("coverage","offtarget")):
					self.appLogger.debug("offtarget")
					offTargetValues=self.parser.get("coverage","offtarget").strip().split(",")
					if not (len(offTargetValues) ==2):
						return False, "\n\t{0}\n\t{1}\n\t{2}\n\t{3}".format(\
							parserMessageWrong,\
							"[coverage] block: offtarget option invalid.",\
							"Incorrect number of parameters.",\
							"Please verify. Exiting"\
							)
					try:
						self.offtarget["loci"]=float(offTargetValues[0])
						self.offtarget["coverage"]=float(offTargetValues[1])
					except:
						return False, "\n\t{0}\n\t{1}\n\t{2}\n\t{3}".format(\
							parserMessageWrong,\
							"[coverage] block: offtarget option invalid.",\
							"One or both values are incorrect.",\
							"Please verify. Exiting"\
							)

					if (self.offtarget["loci"] < 0 or self.offtarget["loci"] > 1) and \
						(self.offtarget["coverage"] < 0 or self.offtarget["coverage"] > 1):
						return False, "\n\t{0}\n\t{1}\n\t{2}\n\t{3}".format(\
							"[coverage] block: offtarget option invalid.",\
							"One or both values are out of range",\
							"Value should be in the interval [0,1]",\
							"Please verify. Exiting"\
							)
					self.ontarget=1-self.offtarget["loci"]
					self.parser.set("coverage","ontarget", str(self.ontarget))

				if (self.parser.has_option("coverage","notcaptured")):
					self.appLogger.debug("not captured")
					self.notcaptured=self.parser.getfloat("coverage","notcaptured")
					if not (self.notcaptured <= 1):
						return False, "\n\t{0}\n\t{1}\n\t{2}\n\t{3}".format(\
							parserMessageWrong,\
							"[coverage] block: notcaptured option invalid.",\
							"Value out of range, should be within the half-closed interval [0,1)",\
							"Please verify. Exiting"\
						)
			else:
				if (self.parser.has_option("coverage","ontarget")): self.parser.remove("coverage","ontarget")
				if (self.parser.has_option("coverage","offtarget")): self.parser.remove("coverage","offtarget")
				if (self.parser.has_option("coverage","notcaptured")): self.parser.remove("coverage","notcaptured")
				if (self.parser.has_option("coverage","ontarget") or self.parser.has_option("coverage","offtarget") or self.parser.has_option("coverage","notcaptured")):
					self.appLogger.warning("Coverage options - offtarget and notcaptured - only possible with [data] inputmode = 4 (gene-tree distribution)")
			# ------------------------------------------------------------------
			#  Coverage distribution related
			if (self.parser.has_option("coverage","experiment")):
				self.appLogger.debug("Coverage experiment")
				value=self.parser.get("coverage","experiment")
				self.parser.set("coverage","experiment",value.lower())
			else:
				# parsear distribution
				parserMessageWrong="\n\t{0}\n\t{1}".format(\
					"[coverage] block:  experiment option is required.",\
					"Please verify. Exiting."\
				)
				return False,parserMessageWrong

			self.experiment=ngsphydistro(self.parser.get("coverage","experiment"), False)
			check,mess=self.experiment.validate()
			if not (check):
				parserMessageWrong=mess
				return check,parserMessageWrong

			# INDIVIDUAL -------------------------------------------------------
			if (self.parser.has_option("coverage","individual")):
				self.appLogger.debug("Coverage individual")
				value=self.parser.get("coverage","individual")
				self.parser.set("coverage","individual",value.lower())
				self.individual=ngsphydistro(self.parser.get("coverage","individual"), False)
				check,mess=self.individual.validate()
				if not (check):
					parserMessageWrong=mess
					return check,parserMessageWrong
			# LOCUS ------------------------------------------------------------
			if (self.parser.has_option("coverage","locus")):
				self.appLogger.debug("Coverage locus")
				value=self.parser.get("coverage","locus")
				self.parser.set("coverage","locus",value.lower())
				self.locus=ngsphydistro(self.parser.get("coverage","locus"), False)
				check,mess=self.locus.validate()
				if not (check):
					parserMessageWrong=mess
					return check,parserMessageWrong
			# ------------------------------------------------------------------
			#  taxon
			if (self.parser.has_option("coverage","taxon")):
				values1=self.parser.get("coverage","taxon").strip().split(";")
				values=[ value.split(",") for value in values1 ]
				if not (len(values)%2==0):
					return False, "\n\t{0}\n\t{1}\n\t{2}\n\t{3}".format(\
						parserMessageWrong,\
						"[coverage] block: taxon option invalid.",\
						"Incorrect number of values.",\
						"Please verify. Exiting"\
						)
				values=sum(values,[])
				taxonOk=True
				for item in xrange(0,len(values),2):
					try:
						val=float(values[item+1])
					except:
						return False, "\n\t{0}\n\t{1}\n\t{2}\n\t{3}".format(\
							parserMessageWrong,\
							"[coverage] block: taxon option invalid.",\
							"One or many values are invalid.",\
							"Please verify. Exiting"\
							)
					if (val >= 0 and val <=1):
						self.taxon[str(values[item])]=val
					else:
						taxonOk=False
				if not taxonOk:
					return False, "\n\t{0}\n\t{1}\n\t{2}\n\t{3}".format(\
						parserMessageWrong,\
						"[coverage] block: taxon option invalid.",\
						"One or many values are out of range. Value should be in the interval [0,1].",\
						"Please verify. Exiting"\
						)
				self.appLogger.info("Applying taxon-specific coverage")
		else:
			# No coverage block
			message="\n\t{0}\n\t{1}\n\t{2}\n\t".format(
				"Settings: [coverage] block",\
				"When using [ngs-reads-art] or [ngs-read-counts] block. Coverage is required.",\
				"Please verify. Exiting."\
			)
			return False,message

		return True, message

	def checkBlockExecution(self,parserMessageCorrect,parserMessageWrong):
		"""
		Checks execution block of the settings file, regarding parameters
		of the general execution of the programs, type, threading...
		------------------------------------------------------------------------
		Returns:
		- boolean, message: the status of the process and the message related to
		such status
		"""
		########################################################################
		# BLOCK: Execution
		########################################################################
		if not self.parser.has_section("execution"):
			self.appLogger.warning("Settings - Execution block: This block has been automatically generated.")
			self.parser.add_section("execution")
			self.parser.set("execution", "environment","bash")
			self.parser.set("execution", "runART","off")
			self.parser.set("execution", "threads","1")
			self.parser.set("execution", "runnint_times","off")
		else:
			####################################################################
			# OPTION: Environment
			if (self.parser.has_option("execution","environment")):
				# got the long name, make sure it is lowercase and within the options
				value=self.parser.get("execution","environment")
				if (value in ["sge","slurm","bash"]):
					self.parser.set("execution","environment",value.lower())
					if value =="bash": self.executionMode=1
					if value =="sge": self.executionMode=2
					if value =="slurm": self.executionMode=3
					if self.executionMode > 1:	self.runART=False
				else:
					message="\n\t{0}\n\t{1}\n\t{2}".format(\
						"Settings: Execution block",\
						"Environment variable is incorrect or unavailable.",\
						"Please check the settings file and rerun. Exiting."
					)
					return False,message
			else:
				message="\n\t{0}\n\t{1}\n\t{2}".format(\
					"Settings: Execution block",\
					"Environment variable is incorrect or unavailable.",\
					"Please check the settings file and rerun. Exiting."
				)
				return False,message

			####################################################################
			# OPTION: RUN
			if (self.parser.has_option("execution","runART")):
				try:
					self.runART=self.parser.getboolean("execution","runART")
				except Exception as e:
					message="\n\t{0}\n\t{1}\n\t{2}".format(\
						"Settings: Execution block",\
						"runART variable is incorrect.",\
						"Please check the settings file and rerun. Exiting."
					)
					return False,message
			else:
				self.appLogger.warning("Settings - Execution block: Run automatically set up to OFF.")
				self.parser.set("execution","runART","off")
			####################################################################
			# OPTION: threads
			if (self.parser.has_option("execution","threads")):
				try:
					self.numThreads=self.parser.getint("execution","threads")
				except Exception as e:
					self.appLogger.warning("Settings - Execution block: Threads automatically set up to 1.")
					self.parser.set("execution","threads","1")
					self.numThreads=1
			else:
				self.numThreads=1
				self.appLogger.warning("Settings - Execution block: Threads automatically set up to 1.")
				self.parser.set("execution","threads","1")
			####################################################################
			 # get running times
 			if (self.parser.has_option("execution","running_times")):
 				try:
 					self.runningTimes=self.parser.getboolean("execution","running_times")
 				except Exception as e:
 					self.runningTimes=False
 			else:
 				self.runningTimes=False
 				self.parser.set("execution","running_times","0")


	def checkIndelibleControlFile(self,parserMessageCorrect,parserMessageWrong):
		"""
		Checks indelible control file according to the modifications for this program.
		------------------------------------------------------------------------
		Returns:
		- boolean, message: the status of the process and the message related to
		such status
		"""
		self.appLogger.debug("checkIndelibleControlFile(self,parserMessageCorrect,parserMessageWrong)")
		# keeping only lines with content
		newlines=None
		with open(self.indelibleControlFile,"r") as f:
			newlines=[ line.strip() for line in f if not line.strip()==""]
		# check for NGSPHY blocks
		model=0; partition=0
		for index in xrange(0,len(newlines)):
			if newlines[index].startswith("[MODEL]") and  model == 0:
				if model== 0:
					model=index
				else:
					parserMessageWrong="Validating INDELible Control file: \n\t{0}\n\t{1}".format(\
						"There is more than one [MODEL] block.",\
						"Please verify. Exiting"\
						)
					return False, parserMessageWrong
			if newlines[index].startswith("[NGSPHYPARTITION]") and partition == 0:
				if partition==0:
					partition=index
				else:
					parserMessageWrong="Validating INDELible Control file: \n\t{0}\n\t{1}".format(\
						"There is more than one [NGSPHYPARTITION] block.",\
						"Please verify. Exiting"\
						)
					return False, parserMessageWrong

		# if i have gotten here and did not return before
		# i can check if the blocks match :D
		# 1) tree filename matches partition parameter and partition has the right
		# number of parameters
		self.partition=newlines[partition].split()
		if len(self.partition)!=4:
			parserMessageWrong="Validating INDELible Control file: \n\t{0}\n\t{1}\n\t{2}".format(\
			"[NGSPHYPARTITION] block has the wrong number of parameters.",\
			"[NGSPHYPARTITION] <tree_filename_basename> <model_name> <sequence_length>"
			"Please verify. Exiting"
			)
			return False, parserMessageWrong
		# check tree corresponds to newick inputbasename
		newickBasename,_=os.path.splitext(os.path.basename(self.geneTreeFile))
		if not self.partition[1]==newickBasename:
			parserMessageWrong="Validating INDELible Control file: \n\t{0}\n\t{1}\n\t{2}\n\t{3}".format(\
			"[NGSPHYPARTITION] block, tree name does not correspond with the Newick File introduced.",\
			"Remember! Newick filename: newick.tree.",\
			"[NGSPHYPARTITION] newick model1 200"
			"Please verify. Exiting"
			)
			return False, parserMessageWrong

		modelname=newlines[model].strip().split()[1]
		if not self.partition[2]==modelname:
			parserMessageWrong="Validating INDELible Control file: \n\t{0}\n\t{1}\n\t{2}\n\t{3}".format(\
			"[NGSPHYPARTITION] block, model name does not correspond to the model defined.",\
			"[MODEL] modelname\n...",\
			"[NGSPHYPARTITION] tree modelname 200"
			"Please verify. Exiting"
			)
			return False, parserMessageWrong

		if not self.partition[3].isdigit():
			parserMessageWrong="Validating INDELible Control file: \n\t{0}\n\t{1}".format(\
			"[NGSPHYPARTITION] block, sequence length is not valid.",\
			"Please verify. Exiting"
			)
			return False, parserMessageWrong

		return True, parserMessageCorrect

	def checkSimPhyProjectValid(self):
		"""
		Verifies that the data given as input from a SimPhy project is complete
		and correct
		------------------------------------------------------------------------
		Returns:
		- boolean, message: the status of the process and the message related to
		such status
		"""
		message=""
		command = os.path.join(self.basepath,"{0}.command".format(self.projectName))
		params = os.path.join(self.basepath,"{0}.params".format(self.projectName))
		db = os.path.join(self.basepath,"{0}.db".format(self.projectName))
		# List all the things in the project directory
		fileList=[os.path.abspath(os.path.join(self.basepath,item)) for item in os.listdir(self.basepath)]
		self.appLogger.debug("SimPhy files (command, params, db)")
		self.appLogger.debug("{0}:\t{1}".format(\
			os.path.basename(db),db in fileList))
		self.appLogger.debug("{0}:\t{1}".format(\
			os.path.basename(command),command in fileList))
		self.appLogger.debug("{0}:\t{1}".format(\
			os.path.basename(params),params in fileList))
		simphyfiles=((command in fileList) and (params in fileList) and(db in fileList))
		# check if  command, db, params files
		if not simphyfiles:
			message="\n\t{0}\n\t{1}".format(\
			"One of the mandatory SimPhy files does not exist.",\
			"Please verify. Exiting.")
			return False, message

		# check how many of them are dirs
		for item in fileList:
			baseitem=os.path.basename(item)
			if (os.path.isdir(os.path.abspath(item)) and  baseitem.isdigit()):
				self.numReplicates=self.numReplicates+1

		self.parser.set("general","numreplicates",str(self.numReplicates))
		# check if at least one
		self.appLogger.debug("Num species trees:\t{0}".format(self.numReplicates))
		if not (self.numReplicates>0):
			message="\n\t{0}\n\t{1}".format(
				"Not enough number of species tree replicates (at least 1 required)",\
				"Please verify. Exiting."\
			)
			return False, message

		return True, message


	def checkBranchLengthsInTree(self):
		"""
		Verifies existence of branch lengths in the given tree
		------------------------------------------------------------------------
		Returns:
		- boolean, message: the status of the process and the message related to
		such status
		"""
		self.appLogger.debug("Checking branch lengths")
		message=""; status=True
		tree=dendropy.Tree.get(path=self.geneTreeFile, schema="newick",preserve_underscores=True)
		leaves=[ node for node in tree.leaf_node_iter()]
		leafedge=None

		for leaf in leaves:
			leafedge= leaf.edge_length
			if not leafedge:
				break
		if not leafedge:
			message="\n\t{0}\n\t{1}\n\t{2}\n\t{3}\n\t{4}\n\t{5}\n\t".format(\
			"INDELible control file - Something's wrong!",\
			"Branch lengths are not specified in the gene tree file.",\
			"Please verify. Exiting."\
			)
			status=False
		return status,message

	def checkLabelFormatInTree(self):
		"""
		Verifies that the data given in gene tree format follows the formatting
		rules given. Specifically, that the labels of the tree are in the
		<SpeciesID_LocusID_IndividualID> format.
		------------------------------------------------------------------------
		Returns:
		- boolean, message: the status of the process and the message related to
		such status
		"""
		self.appLogger.debug("Checking labels")
		message=""
		tree=dendropy.Tree.get(path=self.geneTreeFile, schema="newick",preserve_underscores=True)
		leaves=[ node.taxon.label for node in tree.leaf_node_iter()]
		item=""
		for item in leaves:
			if not  bool(re.match("^([1-9]+_[0-9]+_[0-9]+){1}",item)):
				break
		if not bool(re.match("^([1-9]+_[0-9]+_[0-9]+){1}",item)):
			message="\n\t{0}\n\t{1}\n\t{2}\n\t{3}\n\t{4}\n\t{5}\n\t".format(\
			"INDELible control file - Something's wrong!",\
			"Labels chosen for the tips of the tree are not correct.",\
			"Labels should follow this pattern: SpeciesID_LocusID_IndividualID",\
			"Where SpeciesID,LocusID,IndividualID are numbers.",\
			"Where SpeciesID > 1, LocusID > 0, IndividualID > 0",\
			"Please verify. Exiting."\
			)
			return False,message
		return True, message

	def checkAnchorTipLabelInTree(self):
		"""
		Verifies that the parameter given to select a root sequence is within
		the gene tree given.
		------------------------------------------------------------------------
		Returns:
		- boolean, message: the status of the process and the message related to
		such status
		"""
		# checking if label in the given tree
		self.appLogger.debug("Checking if label in the given tree")
		messageCorrect="Labels of the tree are correct"
		messageWrong="INDELible control file - Something's wrong!\n\t"
		tree=dendropy.Tree.get(path=self.geneTreeFile, schema="newick",preserve_underscores=True)
		filter = lambda taxon: True if taxon.label==self.anchorTipLabel else False
		node = tree.find_node_with_taxon(filter)
		if not node:
			messageWrong+="\n\t{0}\n\t{1}".format(\
			"Tip label for reference sequence is not in the Newick file.",\
			"Please verify. Exiting.")
			return False, messageWrong
		return True, messageCorrect

	def formatSettingsMessage(self):
		"""
		Prints all the settings.
		"""
		message="Settings:\n"
		sections=self.parser.sections()
		for sec in sections:
			message+="\t{0:60s}\n".format("[{}]".format(sec).upper())
			if (sec=="ngs-reads-art"):
				if self.coveragemodeART:
					extra="Number of reads/read pairs to be generated per sequence/amplicon"
					param0="rcount"
				else:
					extra="Fold of read coverage to be simulated or number of reads/read pairs generated for each amplicon"
					param0="fcov"
				param1=True
				message+="\t\t{0:<30s} :\t{1} ({2})\n".format(param0,str(param1).lower(),extra)
			items=self.parser.items(sec)
			for param in items:
				extra=""
				if param[0]=="inputmode":
					if int(param[1])==1: extra="(Single gene tree)"
					if int(param[1])==2: extra="(Single gene tree with ancestral sequence)"
					if int(param[1])==3: extra="(Single gene tree with anchor sequence)"
					if int(param[1])==4: extra="(Gene-tree distribution - SimPhy output)"
				if param[0]=="ploidy":
					if int(param[1])==1: extra="(Haploid individuals)"
					if int(param[1])==2: extra="(Diploid individuals)"
				message+="\t\t{0:<30s} :\t{1} {2}\n".format(param[0],param[1],extra)
		return message

	def correctContentReferenceSequence(self):
		"""
		Verifies content of the reference sequence file for nucleotides only.
		------------------------------------------------------------------------
		Returns:
		- boolean, message: the status of the process and the message related to
		such status
		"""
		self.appLogger.debug("checking content of the refernece sequence")
		status=True; message=""
		# need to read the first line of the file to get the description:
		f=open(self.ancestralSequenceFilePath, "r")
		description=f.readline().strip()
		f.close()
		description=description[1:len(description)]
		referenceDict=msatools.parseMSAFileWithDescriptions(self.ancestralSequenceFilePath)
		reference=referenceDict[description]
		for item in reference:
			if not item.upper() in self.__NUCLEOTIDES:
				status=False
				message="\n\t{0}{1}{2}\n\t{3}\n\t{4}".format(
					"[data] block: Input mode (",self.inputmode,") selected but invalid option.",\
					"Reference sequence should be a nucleotidic sequence, but some other characters exist..",\
					"Please verify. Exiting."\
					)
				break
		return status, message
